/*++ delreq.c - Network Queueing System
 *
 * $Source$
 *
 * DESCRIPTION:
 *
 *	Delete a queued NQS request, or signal a running NQS request.
 *
 *
 *	Author:
 *	-------
 *	Brent A. Kingsbury, Sterling Software Incorporated.
 *	October 3, 1985.
 *
 *
 * STANDARDS VIOLATIONS:
 *   None.
 *
 * REVISION HISTORY: ($Revision$ $Date$ $State$)
 * $Log$
 *
 */

#include "nqspacket.h"			/* NQS local message packets */
#include "transactcc.h"			/* Transaction completion codes */
#if	BSD43 | SGI | SYS52 | SYS53 | ULTRIX
#include <sys/types.h>			/* Uid_t, gid_t */
#else
#if	BSD42 | UNICOS | UTS
#else
BAD SYSTEM TYPE
#endif
#endif
#include NMAP				/* Mid_t (all OS's) */
					/* Uid_t and gid_t (if not BSD43) */

extern long inter();			/* Send local packet */
extern void interclear();		/* Clear packet */
extern void interw32i();		/* Write packet integer datum */

/*** delreq
 *
 *
 *	long delreq():
 *	Delete a queued NQS request, or signal a running NQS request.
 *
 *	Returns:
 *		TCML_INTERNERR:	 if maximum packet size exceeded.
 *		TCML_NOESTABLSH: if unable to establish inter-
 *				 process communication.
 *		TCML_NOLOCALDAE: if the NQS daemon is not running.
 *		TCML_NOSUCHREQ:	 if the specified request does not
 *				 exist.
 *		TCML_NOSUCHSIG:	 if the request was running or in
 *				 transit, and the specified signal
 *				 to send is not recognized by
 *				 the execution machine.
 *		TCML_NOTREQOWN:	 if the specified request is not
 *				 owned by the client.
 *		TCML_PROTOFAIL:	 if a protocol failure occurred.
 *		TCML_REQDELETE:	 if the request was queued, and was
 *				 deleted.
 *		TCML_REQRUNNING: if the request is running, and has
 *				 not been signalled.
 *		TCML_REQSIGNAL:	 if the request was running, and
 *				 has been signalled.
 */
long delreq (mapped_uid, orig_seqno, orig_mid, sig, reqquestate)
uid_t mapped_uid;			/* Mapped owner user-id */
long orig_seqno;			/* Original sequence# */
mid_t orig_mid;				/* Original machine-id of req */
int sig;				/* Signal to send to the processes */
					/* in the req, if running.  0 if no*/
					/* signals are to be sent to */
					/* running request processes */
int reqquestate;			/* Request queue state */
{
	interclear();
	interw32i ((long) mapped_uid);
	interw32i ((long) orig_seqno);
	interw32i ((long) orig_mid);
	interw32i ((long) sig);
	interw32i ((long) reqquestate);
	return (inter (PKT_DELREQ));
}

 
/*** diagqdel
 *
 *
 *	diagdel():
 *	Diagnose delreq() completion code.
 */
int diagdel (code, reqid)
long code;				/* Transaction completion code */
char *reqid;				/* Character version of request-id */
{
	switch (code) {
	case TCML_INTERNERR:
		printf ("Internal error.\n");
		exiting();		/* Delete communication file */
		exit (-1);
	case TCML_NOESTABLSH:
		printf ("Unable to establish inter-process communications ");
		printf ("with NQS daemon.\n");
		printf ("Seek staff support.\n");
		exiting();		/* Delete communication file */
		exit (-1);
	case TCML_NOLOCALDAE:
		printf ("The NQS daemon is not running.\n");
		printf ("Seek staff support.\n");
		exiting();		/* Delete communication file */
		exit (-1);
	case TCML_NOSUCHREQ:
		printf ("Request %s does not exist.\n", reqid);
		break;
	case TCML_NOSUCHSIG:
		printf ("Request %s is running, and the ", reqid);
		printf ("specified signal is\n");
		printf ("not recognized by the execution machine.\n");
		break;
	case TCML_NOTREQOWN:
		printf ("Not owner of request %s.\n", reqid);
		break;
	case TCML_PEERDEPART:
		printf ("Request %s is presently being routed by a ", reqid);
		printf ("pipe queue,\n");
		printf ("and this NQS implementation does not support the ");
		printf ("deletion of a\n");
		printf ("request under such conditions.\n");
		break;
	case TCML_PROTOFAIL:
		printf ("Protocol failure in inter-process communications ");
		printf ("with NQS daemon.\n");
		printf ("Seek staff support.\n");
		exiting();		/* Delete communication file */
		exit (-1);
	case TCML_REQDELETE:
		printf ("Request %s has been deleted.\n", reqid);
		break;
	case TCML_REQRUNNING:
		printf ("Request %s is running.\n", reqid);
		break;
	case TCML_REQSIGNAL:
		printf ("Request %s is running, and has ", reqid);
		printf ("been signalled.\n");
		break;
	default:
		printf ("Unexpected completion code from NQS daemon.\n");
		exiting();		/* Delete communication file */
		exit (-1);
	}
	return (0);
}
