/*++ fetchpwdb.c - Network Queueing System
 *
 * $Source$
 *
 * DESCRIPTION:
 *
 *	This module contains three functions:
 *
 *		fetchpwuid(),
 *		fetchpwnam(), and
 *		closepwdb()
 *
 *	that respectively return the passwd structure for the specified
 *	local user-id, return the passwd structure for the specified
 *	local username, and close the local account/password database
 *	opened by fetchpwuid() or fetchpwnam().
 *
 *	This module is provided as an "onion-skin layer" residing above
 *	the locally implemented account/password database access
 *	functions.
 *
 *
 *	Author:
 *	-------
 *	Brent A. Kingsbury, Sterling Software Incorporated.
 *	January 20, 1986.
 *
 *
 * STANDARDS VIOLATIONS:
 *   None.
 *
 * REVISION HISTORY: ($Revision$ $Date$ $State$)
 * $Log$
 *
 *	August 21, 1991	Performance reasons : call getpwuid() or getpwnam()
 *			instead of getpwent()
 *
 */

#include <stdio.h>			/* for definition of L_cuserid */
#include <pwd.h>			/* Password file structures */


/*
 *	External functions.
 */
extern void endpwent();			/* Close password file */
extern struct passwd *getpwuid();	/* Read password file entry for uid */
extern struct passwd *getpwnam();	/* Read password file entry for name */


/*** fetchpwuid
 *
 *
 *	struct passwd *fetchpwuid():
 *	Return struct passwd structure for the specified local user-id.
 *
 *	NOTE:	This function does not open and close the
 *		account/password database on successive
 *		invocations.  It is up to the caller to
 *		invoke closepwdb() when access to the
 *		account/password database is no longer
 *		required.
 */
struct passwd *fetchpwuid (uid)
register int uid;			/* User-id to be identified */
{
	return (getpwuid (uid));
}


/*** fetchpwnam
 *
 *
 *	struct passwd *fetchpwnam():
 *	Return struct passwd structure for the specified local username.
 *
 *	NOTE:	This function does not open and close the
 *		account/password database on successive
 *		invocations.  It is up to the caller to
 *		invoke closepwdb() when access to the
 *		account/password database is no longer
 *		required.
 */
struct passwd *fetchpwnam (name)
register char *name;
{
	return (getpwnam(name));
}


/*** closepwdb
 *
 *
 *	void closepwdb():
 *
 *	Close the local account/password database opened by fetchpwuid(),
 *	or by fetchpwnam().
 */
void closepwdb()
{
	endpwent();			/* Close the local password file */
}
