/*++ getsockch.c - Network Queueing System
 *
 * $Source$
 *
 * DESCRIPTION:
 *
 *	Routines for reading from a socket.
 *
 *
 *	Author:
 *	-------
 *	Brent A. Kingsbury, Sterling Software Incorporated.
 *	April 21, 1986.
 *
 *
 * STANDARDS VIOLATIONS:
 *   None.
 *
 * REVISION HISTORY: ($Revision$ $Date$ $State$)
 * $Log$
 *
 */

#include <stdio.h>
#include "nqs.h"			/* NQS types and definitions */
					/* + sys/types.h + nmap.h + quolim.h */
#include <errno.h>

/*
 *	External variables.
 */
extern int errno;			/* System call error codes */

/*
 *	Internal variables.
 */
static int sockfd;			/* Socket file descriptor */
static char buffer [MAX_PACKET];	/* Receive buffer */
static int bufptr = 0;			/* Index of next char in buffer */
static int n_read = 0;			/* Index of 1st empty slot in buffer */


/*** bufsockch
 *
 *
 *	char *bufsockch():
 *
 *	Return a pointer to the characters that
 *	1) this module has read from the socket, and
 *	2) have not yet been turned over with getsockch().
 *
 */
char *bufsockch ()
{
	return (&buffer[bufptr]);
}


/*** extrasockch
 *
 *
 *	int extrasockch():
 *
 *	Report the number of characters that
 *	1) this module has read from the socket, and
 *	2) have not yet been turned over with getsockch().
 */
int extrasockch ()
{
	return (n_read - bufptr);
}


/*** getsockch
 *
 *
 *	int getsockch():
 *
 *	Return a character from a socket.
 */
int getsockch ()
{
	register int no_packet_yet;

	if (bufptr < n_read) {
		return (buffer [bufptr++] & 0377);
	}
	bufptr = 1;			/* Return buffer [1] NEXT time */
	no_packet_yet = 1;
	do {
		if ((n_read = read (sockfd, buffer, MAX_PACKET)) == -1) {
			if (errno != EINTR) {
				return (-1);
			}
		}
		else no_packet_yet = 0;
	} while (no_packet_yet);
	if (n_read == 0) {
		return (-1);
	}
	return (buffer [0] & 0377);
}


/*** setsockfd
 *
 *
 *	void setsockfd():
 *
 *	Notify this module which socket it is that it should read on.
 */
void setsockfd (fd)
int fd;					/* File descriptor */
{
	sockfd = fd;
}
