/*++ getwdir.c - Network Queueing System
 *
 * $Source$
 *
 * DESCRIPTION:
 *
 *
 *	Get current working directory (that should work on both
 *	System V and Berkeley 4.2 systems).  Note that things
 *	certainly would have been easier, if we had just used the
 *	standard getcwd() call of System V.  The only problem is
 *	that the System V version of getcwd() is implemented by
 *	using popen() to pipe the output of the pwd command into
 *	a returned buffer space, and I got tired of waiting so
 *	long.
 *
 *
 *	Author:
 *	-------
 *	Brent A. Kingsbury, Sterling Software Incorporated.
 *	August 12, 1985.
 *
 *
 * STANDARDS VIOLATIONS:
 *   None.
 *
 * REVISION HISTORY: ($Revision$ $Date$ $State$)
 * $Log$
 *
 *	July 11, 1991	Apollo AEGIS system call for performance reasons
 *
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#if	SYS52 | SYS53 | UNICOS
#include <dirent.h>			/* Directory walking */
#else
#ifdef	UTS
#include "dir.4.2.h"			/* Directory walker emulations */
#else
#if BSD42 | BSD43 | SGI | SYS52 | SYS53 | ULTRIX
#include <sys/dir.h>			/* Directory walking */
#else
BAD SYSTEM TYPE
#endif
#endif
#endif
#define	MAXPATHLEN	1500		/* Maximum path length possible */
					/* for getworkdir()  */

extern int strlen();

/*** getwdir
 *
 *
 *	char getwdir():
 *
 *	Returns:
 *		A pointer to the fully qualified path name of the
 *		current working directory if successful;
 *		Otherwise, NULL is returned (an error occurred).
 *
 *		WARNING:
 *			If NULL is returned indicating that we were
 *			unsuccessful in determining the current working
 *			directory, then the current working directory of
 *			the caller may, or may NOT be the same directory
 *			as when this function was invoked!!!!!
 *
 *			In other words, if this function ever returns
 *			NULL, then the caller should bail out.
 */
char *getwdir ()
{
	static char path [MAXPATHLEN + 1];
	struct stat child;
	struct stat parent;
	struct stat walkparent;
	DIR *dir;
#if	SYS52 | SYS53 | UNICOS
	struct dirent *dirent;
#else
	struct direct *dirent;
#endif
	register char *ptrpath;
	register int i;

#ifdef	DOMAIN_OS
#include <apollo/base.h>
#include <apollo/name.h>			/* ANSI option required ! */

unsigned short path_length;
status_$t status;

	name_$get_wdir_lc ((unsigned short)MAXPATHLEN, path, &path_length,
		&status);
	if (status.all)
		return (NULL);
	return (path);
#else
#if	BSD43 | SGI | SYS53 | ULTRIX
#ifdef	NEXT
	return ((char *)getwd (path));
#else
	return ((char *)getcwd (NULL, MAXPATHLEN));
#endif
#else
	ptrpath = path + MAXPATHLEN;
	*ptrpath-- = '\0';			/* Null terminate */
	if (stat (".", &child) == -1) {
		/* stat() failed */
		return (NULL);
	}
	for(;;) {
		if ((dir = opendir ("..")) == NULL) {
			/* opendir() failed */
			if (*++ptrpath != '\0') chdir (++ptrpath);
			return (NULL);
		}
		if (fstat (dir->dd_fd, &parent) == -1) {
			/* fstat() failed */
			if (*++ptrpath != '\0') chdir (++ptrpath);
			return (NULL);
		}
		if (chdir ("..") == -1) {
			/*  chdir() failed. */
			if (*++ptrpath != '\0') chdir (++ptrpath);
			return (NULL);
		}
		if (child.st_dev == parent.st_dev) {
			/*
			 *  The parent directory is on the same device as
			 *  the child directory.
			 */
			if (child.st_ino == parent.st_ino) {	/* At root */
				/*
				 *  Restore to the original working directory
				 *  and return to caller.
				 */
#ifdef	DOMAIN_OS
				*ptrpath = '/';
				if (chdir (ptrpath) == -1) return (NULL);
#else
				if (*++ptrpath == '\0') {
					/*
					 *  We were at '/'.
					 */
					*--ptrpath = '/';
				}
				else if (chdir (ptrpath) == -1) return (NULL);
#endif
				return (ptrpath);
			}
			do {	/* Loop to locate parent's name */
				if ((dirent = readdir (dir)) == NULL) {
					/* Read error */
					if (*++ptrpath != '\0') {
						chdir (++ptrpath);
					}
					return (NULL);
				}
			} while (dirent->d_ino != child.st_ino);
		}
		else {
			/*
			 *  The parent directory is on a different device
			 *  than the child directory.
			 */
			do {
				if ((dirent = readdir (dir)) == NULL) {
					/* Read error */
					if (*++ptrpath != '\0') {
						chdir (++ptrpath);
					}
					return (NULL);
				}
				stat (dirent->d_name, &walkparent);
			} while (walkparent.st_ino != child.st_ino ||
				 walkparent.st_dev != child.st_dev);
		}
		closedir (dir);
		i = strlen (dirent->d_name);
		if (ptrpath - path < i) {
			/*
			 *  The full path name is simply too long.
			 */
			if (*++ptrpath != '\0') chdir (++ptrpath);
			return (NULL);
		}
		while (i > 0) *ptrpath-- = dirent->d_name [--i];
		*ptrpath-- = '/';
		/*
		 *  We did a chdir() to the parent directory, so now
		 *  the child should be the parent here for the next
		 *  iteration of the loop.
		 */
		child = parent;
	}
#endif
#endif
}
