#include <stdio.h>
#include "nqs.h"
#if	UNICOS
#include <fcntl.h>			/* File control */
#include <nlist.h>
#include <sys/param.h>
#include <sys/jtab.h>
#else
#if	SGI | SYS52 | SYS53 | UTS
#include <fcntl.h>			/* File control */
#else
#if	BSD42 | BSD43 | ULTRIX
#include <sys/file.h>
#else
BAD SYSTEM TYPE
#endif
#endif
#endif
#include <pwd.h>
#include "nqsxdirs.h"			/* NQS files and directories */

#define DSP_ERR1	"-u and -A are mutually exclusive"
#define DSP_ERR2	" or one has been used previously in the command."
#define DSP_ERR3	"multiple uses of "
#define DSP_ERR4	"\tadministrator option '-u or -A' was"
#define DSP_ERR5	"requested by an invalid NQS manager"


/*
 *	External functions:
 */
extern void bufstdout();		/* Buffer stdout */
extern void closepwdb();		/* Close the password file */
extern int daepres();			/* Determine if the local NQS */
					/* daemon is present */
extern struct passwd *fetchpwnam();	/* NQS version of getpwnam() */
extern uid_t getuid();			/* Get real user-id */
extern struct confd *opendb();		/* Open an NQS database file */
extern int shoqbyname();		/* Show a queue given by name */

/*
 *	Global variables:
 */
char *Qstat_prefix = "Qstat";
int scpflag = 0;
#if	UNICOS
struct	jtab	jtab[NJOB];	/* job table */
#endif

/*** main_dsp
 *
 * set up everything for the displays
 * open needed databases, set up special
 * needs, ect.
 */
main_dsp (argc,argv,flags,selector,tname,hname)
int 	argc;
char 	*argv[];
long	flags;
char 	* selector;
char	* tname;
char	* hname;
{
	extern char	*optarg;
	extern int 	optind;

	struct confd *queuefile;	/* Queue description file */
	struct confd *qcomplexfile;	/* Queue complex definition file */
	struct confd *qmapfile;		/* Qmap description file */
	struct confd *pipeqfile;	/* Pipeqfile description file */
	char *argument;
	short daepresent;		/* Boolean non-zero if the local */
					/* NQS daemon is present and running */
	char whom[10];			/* Whom we are interested in */
	struct passwd *whompw;		/* Password entry of "whom" */
	uid_t whomuid;			/* Uid of whom we are interested in */
	mid_t	Local_mid,itsmid;
	int	i;

	/* step 1. change to root directory */
	if (chdir (Nqs_root) == -1) {
		fprintf (stderr, "%s(FATAL): Unable to chdir() to the NQS ",
			 Qstat_prefix);
		fprintf (stderr, "root directory.\n");
		return (-1);
	}

	/* step 2. opend queue, complex, map, and pipeto databases */
	if ((queuefile = opendb (Nqs_queues, O_RDONLY)) == NULL) {
		fprintf (stderr, "%s(FATAL): Unable to open the NQS queue ",
			 Qstat_prefix);
		fprintf (stderr, "definition database file.\n");
		return (-1);
	}

        if ((qcomplexfile = opendb (Nqs_qcomplex, O_RDONLY)) == NULL) {
                fprintf (stderr, "%s(FATAL): Unable to open the NQS queue ",
                         Qstat_prefix);
                fprintf (stderr, "complex definition database file.\n");
                exit (2);
        }
        if ((qmapfile = opendb (Nqs_qmaps, O_RDONLY)) == NULL) {
                fprintf (stderr, "%s(FATAL): Unable to open the NQS queue ",
                         Qstat_prefix);
                fprintf (stderr, "complex definition database file.\n");
                exit (2);
        }
        if ((pipeqfile = opendb (Nqs_pipeto, O_RDONLY)) == NULL) {
                fprintf (stderr, "%s(FATAL): Unable to open the NQS queue ",
                         Qstat_prefix);
                fprintf (stderr, "complex definition database file.\n");
                exit (2);
        }



	/* get user id */
	whomuid = getuid();		/* Assume asking about self */
	if ((whompw=getpwuid(whomuid)) == (struct passwd *) 0) {
		fprintf (stderr, "Who is user %d?\n", whomuid);
		exit (3);
	}
	strcpy(whom,whompw->pw_name);
	strcat(whom,"\0");
	closepwdb();		/* Close the password file */

#if UNICOS
	/* step 6.  Read job table from kernel memory.  */
	read_j_table();
#endif
	/* step 7.  Now, buffer stdout for speed.  */
	bufstdout();


	/* step 8. move to correct arg */
	for(i=0; i<argc; i++)
		argv++;

	/* step 9. if HOST then send to information to host */
	if (hname != NULL) {
		if (get_host_id(  hname, &itsmid) != SUCCESS) {
			printf("host not available. \n");
			exit (0);
		}
		do {
			call_host(whomuid, flags, itsmid, *argv);
			if (*argv != NULL)
				argv++;
		} while (*argv != NULL);
		closedb(qcomplexfile);
		closedb(queuefile);
		closedb(qmapfile);
		closedb(pipeqfile);
		return(0);
	}


	/* step 10. if requested adminstrator flag validate adminstrator 
	 *	if not requeusting adminstrator option then set
	 *	ourself as the selector for requests to be seen
	 */
	if (selector != NULL) {
		if (localmid (&Local_mid) != 0) {
			printf("Unable to get machine-id of local host.\n");
			exit(2);
		}
		if (nqspriv(whomuid,Local_mid,Local_mid) == 0) {
			printf("%s %s \n",DSP_ERR4, DSP_ERR5);
			return(-2);
		}
	}
	else
		selector = whom;



	/* step 11. print requests */
	do {
		list(queuefile, qcomplexfile, *argv, flags, selector,qmapfile,
			pipeqfile);
		if (*argv != NULL)
			argv++;
	} while (*argv != NULL);


	/* step 12.  Flush any output buffers and exit.  */
	fflush (stdout);
	fflush (stderr);
	closedb(qcomplexfile);
	closedb(queuefile);
	closedb(qmapfile);
	closedb(pipeqfile);
	return (0);
}


/*** read_j_table 
 * only for UNICOS needs
 */
read_j_table()
{
#if UNICOS
                register int fd;
                struct nlist nl[2];

                nl[0].n_name = "jtab";
                nl[1].n_name = "";
                fd = open("/dev/kmem", 0);
                if (fd < 0 ||
                    nlist( "/unicos", nl ) < 0 ||
                    lseek( fd, nl[0].n_value, 0 ) < 0 ||
                    read( fd, (char*)jtab, sizeof(jtab) ) != sizeof(jtab)) {
                        fprintf(stderr, "Unable to read job table.\n");
                }
                close(fd);
#endif
}


/*** set_sflag
 *
 * show specific queues that are in specific state
 */
set_sflag(optarg)
char * optarg;
{
	long flags = 0;

	for (; (*optarg != NULL); optarg++) {
		switch(*optarg) {
		case 'h':
			flags |= SHO_RS_HOLD;
			break;
		case 'r':
			flags |= SHO_RS_RUN;
			break;
		case 'q':
			flags |= SHO_RS_QUEUED;
			break;
		case 't':
			flags |= SHO_RS_EXIT;
			break;
		default:
			printf ("Invalid argument specified with option ");
			printf ("'-s'.\n");
			exit (2);
		}	/* end switch */
	}		/* end for    */
	return(flags);
}


/***parse_args
 *
 * parse args for commands
 */
long 
parse_args(argc, argv, string,selector,tname,hname)
  int  *argc;
  char *argv[];
  char * string;
  char ** selector;
  char ** tname;
  char ** hname;
{
	extern 		char	*optarg;
	extern 		int 	optind;
	register	int	c;
	register	int	i;
	register	long	flags=0;
	register	int	state=0;

	while ((c=getopt(*argc, argv, string)) !=-1) {
		switch (c) {
#if UNICOS
		case '*':
			scpflag = strtol(++argument, &argument, 10);
			argument--;
			break;
#endif
		case 'b':		/* batch q */
			flags |= SHO_BATCH;
			break;
		case 'p':		/* pipe q */
			flags |= SHO_PIPE;
			break;
		case 'd':		/* device q */
			flags |= SHO_DEVICE;
			break;
		case 'f':		/* full display */
		case 'l':		/* long display */
			flags |= SHO_FULL;
			break;
		case 'L':		/* rescources */
			flags |= SHO_LIMITS;
			break;
		case 'n':		/* no header/trailer */
			flags |= SHO_NOHDR;
			break;
		case 'x':		/* extended by vendor */
			flags |= SHO_EXTEND;
			break;
		case 'h':		/* extended by vendor */
			if (flags & SHO_HOST){
				printf(" %s -h \n",DSP_ERR3);
				exit(2);
			}
			flags |= SHO_HOST;
			*hname = optarg;
			break;
		case 's':		/* extended by vendor */
			state++;
			flags |= set_sflag(optarg);
			break;
		case 'T':		/*target uname in conjunction w/ host*/
			if (flags & SHO_TARGET){
				printf(" %s -T \n",DSP_ERR3);
				exit(2);
			}
			flags |= SHO_TARGET;
			*tname = optarg;
			break;
		case 'A':		/* account list */
			if ((flags & SHO_GROUP) || (flags & SHO_ACCT)
			|| (flags & SHO_USER) || (flags & SHO_R_ALLUID)) {
				printf(" %s \n", DSP_ERR1);
				printf(" %s \n", DSP_ERR2);
				exit(2);
			}
			flags |= SHO_ACCT;
			*selector = optarg;
			break;
		case 'g':		/* group list */
			if ((flags & SHO_GROUP) || (flags & SHO_ACCT)
			|| (flags & SHO_USER) || (flags & SHO_R_ALLUID)) {
				printf(" %s \n", DSP_ERR1);
				printf(" %s \n", DSP_ERR2);
				exit(2);
			}
			flags |= SHO_GROUP;
			*selector = optarg;
			break;
		case 'u':		/* user list */
			if ((flags & SHO_GROUP) || (flags & SHO_ACCT)
			|| (flags & SHO_USER) || (flags & SHO_R_ALLUID)) {
				printf(" %s \n", DSP_ERR1);
				printf(" %s \n", DSP_ERR2);
				exit(2);
			}
			flags |= SHO_USER;
			*selector = optarg;
			break;
		case 'a':		/* show all */
		case 'U':		/* Unrestricted display */
			if ((flags & SHO_GROUP) || (flags & SHO_ACCT)
			|| (flags & SHO_USER)) {
				printf(" %s \n", DSP_ERR1);
				printf(" %s \n", DSP_ERR2);
				exit(2);
			}
			flags |= SHO_R_ALLUID;
			break;
		case 'Q':		/* queue list */
			flags |= (SHO_BATCH | SHO_QUE);
			break;
		case '?':
			fprintf (stderr, "Invalid option flag ");
			fprintf (stderr, "specified.\n");
			fprintf (stderr, "Command format is:\n\n");
			fprintf (stderr, "    %s %s",argv[0],string);
			fprintf (stderr, "[ <list-name(s)> ]\n\n");
			exit (7);
		}
	}

	/* step 5. move argv up to where we stopped parsing */
	*argc=optind;

	/* if no specific type is mentioned then show all types */

	/* Well, on second thought, the default case will be to show */
	/* requests, and that will be taken care of by qstat.c. TS   */
        /*
	 * if ( (! (flags & SHO_BATCH)) && (! (flags & SHO_PIPE)) && 
	 *      (! (flags & SHO_DEVICE)) )
	 * {
	 *      flags |= ( SHO_BATCH | SHO_PIPE | SHO_DEVICE );
	 * }
	 */

	/* step 6. no special state so show all */
	if (state == 0)
		flags |= (SHO_RS_RUN | SHO_RS_EXIT | SHO_RS_HOLD |
			  SHO_RS_QUEUED);

	return(flags);
}
