/*++ mapuser.c - Network Queueing System
 *
 * $Source$
 *
 * DESCRIPTION:
 *
 *	Given a user's identity on another host, return
 *	a pointer to that user's password file entry on this host.
 *
 *
 *	Author:
 *	-------
 *	Robert W. Sandstrom, Sterling Software Incorporated.
 *	May 9, 1986.
 *
 *
 * STANDARDS VIOLATIONS:
 *   None.
 *
 * REVISION HISTORY: ($Revision$ $Date$ $State$)
 * $Log$
 *
 *	May 24, 1991	Allow network access for root accounts
 *	November 15, 1991 Change the name of the access equivalency file
 *	February 6, 1992 Skip blank characters in user .rhosts file
 *
 */

#include <stdio.h>
#include <pwd.h>
#include NMAP				/* Mid_t (all OS's) */
					/* Uid_t and gid_t (if not BSD43) */
#include <sys/types.h>
#include <sys/stat.h>

extern struct passwd *fetchpwnam();	/* NQS version of getpwnam() */
extern struct passwd *fetchpwuid();	/* NQS version of getpwuid() */
extern int nmap_get_uid();		/* Get uid on this machine */
extern char *strcat();			/* Concatenate strings */
extern char *strtok();			/* Locate token in string */
#if	BSD42 | BSD43 | ULTRIX
extern char *index();			/* Locate character in string */
#else
#if	SGI | SYS52 | SYS53 | UNICOS | UTS
extern char *strchr();			/* Locate character in string */
#else
BAD SYSTEM TYPE
#endif
#endif
extern int strcmp();			/* String comparison */
extern char *strcpy();			/* String copy */
extern char *strpbrk();			/* Locate character in string */

/*** mapuser
 *
 *
 *	struct passwd *mapuser: Given a user's identity on another host,
 *	return a pointer to that user's password file entry on this host.
 *
 */
struct passwd *mapuser (fromuid, fromusername, frommid, fromhostname)
uid_t fromuid;				/* Client's uid on remote machine */
char *fromusername;			/* Client username on remote machine */
mid_t frommid;				/* Nmap machine id of remote machine */
char *fromhostname;			/* PRINCIPAL name of remote machine */
{
	uid_t touid;			/* Mapped user-id */
	char dotrhostspath [256];	/* Users .rhost pathname */
	register struct passwd *passwd;	/* Password entry for account */
	short mapbyuid;			/* Boolean */

#ifndef	MAPBYUID
#define MAPBYUID 0			/* Default mapping by username */
#endif

	if (MAPBYUID) {
		/*
		 *  The selected mapping algorithm operates by user-id.
		 */
		switch (nmap_get_uid (frommid, fromuid, &touid)) {
		case NMAP_SUCCESS:
		case NMAP_DEFMAP:
			return (fetchpwuid (touid));
		}
		return ((struct passwd *) 0);
	}
	/*
	 *  The selected mapping algorithm operates by name.
	 */
	if ((passwd = fetchpwnam (fromusername)) == (struct passwd *) 0) {
		/*
		 *  The remote user has no account on this
		 *  machine (or the account names are different).
		 */
		return ((struct passwd *) 0);
	}
	if (fromuid != 0 && strcmp (fromusername, "root")) {
		/*
		 *  We are not mapping root.
		 *  Examine the /etc/hosts.nqs file for machine
		 *  equivalency.
		 */
		if (access ("/etc/hosts.nqs", fromhostname,
			   (struct passwd *) 0)) {
			return (passwd);	/* Access allowed */
		}
	}
	/*
	 *  Determine if the user's account specifically allows access.
	 */
	strcpy (dotrhostspath, passwd->pw_dir);
	if (!strcmp (dotrhostspath, "/"))	/* Root accounts */
		strcat (dotrhostspath, ".rhosts");
	else
		strcat (dotrhostspath, "/.rhosts");
	if (access (dotrhostspath, fromhostname, passwd)) {
		return (passwd);		/* Access allowed */
	}
	return ((struct passwd *) 0);
}


/*** access
 *
 *
 *	int access():
 *
 *	Return non-zero if the specified Berkeley style access file
 *	grants access to the specified remote account.
 *
 *	If the password entry parameter:  passwd  is non-null, then
 *	the access equivalency file being scanned MUST contain lines
 *	of the form:
 *
 *		machinename username
 *
 *	and any such access equivalency file MUST be owned by the
 *	account identified by the given password entry.
 */
static int access (accessfname, fromhostname, passwd)
char *accessfname;			/* Access file name */
char *fromhostname;			/* Name of client machine */
register struct passwd *passwd;		/* Password file entry */
{
	struct stat statbuf;		/* Stat() buffer */
	char accesshost [256];		/* Access file line buffer */
	register FILE *accessfile;	/* Access file */
	register char *cp;
	register char *user;

	accessfile = fopen (accessfname, "r");
	if (passwd != (struct passwd *) 0) {
		/*
		 *  Verify ownership of the .rhosts file, and ensure
		 *  that (on Berkeley systems), that the file is not
		 *  a symbolic link.
		 */
		if (accessfile == (FILE *) 0 ||
			fstat (fileno (accessfile), &statbuf) == -1) return (0);
#if	SGI | SYS52 | SYS53 | UNICOS | UTS
		if (statbuf.st_uid != passwd->pw_uid) return (0);
#else
#if	BSD42 | BSD43 | ULTRIX
		if (statbuf.st_uid != passwd->pw_uid ||
		   (statbuf.st_mode & S_IFMT) == S_IFLNK) {
			return (0);
		}
#else
BAD SYSTEM TYPE
#endif
#endif
	}
	if (accessfile != (FILE *) 0) {
		/*
		 *  The access file was successfully opened.
		 */
		while (fgets (accesshost, sizeof (accesshost),
			      accessfile) != (char *) 0) {
			/*
			 *  We have an access file line to interpret.
			 */
#if	BSD42 | BSD43 | ULTRIX
			cp = index (accesshost, '\n');
#else
#if	SGI | SYS52 | SYS53 | UNICOS | UTS
			cp = strchr (accesshost, '\n');
#else
BAD SYSTEM TYPE
#endif
#endif
			if (cp != (char *) 0) *cp = '\0';
			if (passwd != (struct passwd *) 0) {
				/*
				 *  We are inspecting a user account
				 *  .rhost file.  A username field may not be
				 *  required.
				 */
				if ((user = strpbrk (accesshost, " \t")) ==
						(char *) 0) {
					if (!strcmp (accesshost, fromhostname))
					/*
					 *  Client account is at an
					 *  equivalent host.
				 	 */
						return (1);
					else
						continue;
					
				} else *user++ = '\0';
				/*
				 *  A username field is present (as required);
				 *  skip blank characters.
				 */
				while ((*user == ' ') || (*user == '\t'))
					user++;
				if ((cp = strpbrk (user, " \t")) !=
					(char *) 0) *cp = '\0';
				if (!strcmp (accesshost, fromhostname) &&
				    !strcmp (user, passwd->pw_name)) {
					return (1);
				}
			}
			else {
				if ((cp = strpbrk (accesshost, " \t")) !=
					(char *) 0) *cp = '\0';
				if (!strcmp (accesshost, fromhostname))
				/*
				 *  Client account is at an equivalent host.
				 */
					return (1);
			}
		}
	}
	return (0);			/* No access */
}
