/*++ movreq.c - Network Queueing System
 *
 * $Source$
 *
 * DESCRIPTION:
 *
 *	Send a move request or a move queue packet to the NQS daemon.
 *
 *
 *	Author:
 *	-------
 *	Brent A. Kingsbury, Sterling Software Incorporated.
 *	December 2, 1985.
 *
 *
 * STANDARDS VIOLATIONS:
 *   None.
 *
 * REVISION HISTORY: ($Revision$ $Date$ $State$)
 * $Log$
 *
 *	October 9, 1991	Move requests between queues
 *
 */

#include "nqspacket.h"			/* NQS local message packets */
#if	BSD43 | SGI | SYS52 | SYS53 | ULTRIX
#include <sys/types.h>			/* Uid_t, gid_t */
#else
#if	BSD42 | UNICOS | UTS
#else
BAD SYSTEM TYPE
#endif
#endif
#include NMAP				/* Mid_t (all OS's) */
					/* Uid_t and gid_t (if not BSD43) */

extern long inter();			/* Send local packet */
extern void interclear();		/* Clear packet */
extern void interw32i();		/* Write packet integer datum */
extern void interwstr();		/* Write packet string datum */

/*** movreq
 *
 *
 *	long movreq():
 *	Send a move request packet to the NQS daemon.
 *
 *	Returns:
 *		TCML_COMPLETE:	 if the specified request was
 *				 successfully placed in the specified
 *				 queue.
 *		TCML_INTERNERR:	 if maximum packet size exceeded.
 *		TCML_NOESTABLSH: if unable to establish inter-
 *				 process communication.
 *		TCML_NOLOCALDAE: if the NQS daemon is not running.
 *		TCML_NOSUCHQUE:	 if the specified from or destination
 *				 queue does not exist.
 *		TCML_NOSUCHREQ:	 if the specified request does not
 *				 exist.
 *		TCML_NOTREQOWN:	 if the specified request is not
 *				 owned by the client.
 *		TCML_PROTOFAIL:	 if a protocol failure occurred.
 */
long movreq (orig_seqno, orig_mid, dest_queue_name)
long orig_seqno;			/* Original sequence# */
mid_t orig_mid;				/* Original machine-id of req */
char *dest_queue_name;			/* Name of destination queue for */
{					/* the request */
	interclear();
	interw32i ((long) orig_seqno);
	interw32i ((long) orig_mid);
	interwstr (dest_queue_name);
	return (inter (PKT_MOVREQ));
}


/*** movque
 *
 *
 *	long movque():
 *
 *	Send to the NQS daemon a packet which will move a queue.
 *
 *	Returns:
 *		TCML_COMPLETE:	if successful;
 *		TCML_NOSUCHQUE:	if a named queue does not exist.
 */
long movque (src_qname,des_qname)
char *src_qname;				/* Name of source queue */
char *des_qname;				/* Name of destn queue */
{
	interclear();
	interwstr (src_qname);
	interwstr (des_qname);
	return (inter (PKT_MOVQUE));
}
