/*++ nextseqno.c - Network Queueing System
 *
 * $Source$
 *
 * DESCRIPTION:
 *
 *	Return the next available NQS request sequence number.
 *	The current directory of the caller MUST be the NQS
 *	root directory.
 *
 *	For the reasons behind the peculiar storage of the
 *	NQS request sequence number, see ../src/nqs_ldconf.c.
 *
 *
 *	Author:
 *	-------
 *	Brent A. Kingsbury, Sterling Software Incorporated.
 *	March 20, 1986.
 *
 *
 * STANDARDS VIOLATIONS:
 *   None.
 *
 * REVISION HISTORY: ($Revision$ $Date$ $State$)
 * $Log$
 *
 */

#include "../h/nqsxdirs.h"		/* Get Nqs_seqno definition */
#include <sys/types.h>			/* For sys/stat.h */
#include <sys/stat.h>

/*
 *	LMTOFFSET is 0 on all machines except on Apollo where utimes
 *	does not set modifications times =< 1 Jan 1980
 */
#ifdef	DOMAIN_OS
#define LMTOFFSET	315532801L
#else
#define LMTOFFSET	0L
#endif

/*** nextseqno
 *
 *
 *	long nextseqno():
 *
 *	Return the next available NQS request sequence number.
 *	The current directory of the caller MUST be the NQS
 *	root directory.
 */
long nextseqno ()
{
	struct stat stat_buf;		/* Stat() buffer */

	if (stat (Nqs_seqno, &stat_buf) == -1) return (0);
	if ((stat_buf.st_mode & 0777) == 0) return (0);
	return ((long) stat_buf.st_mtime - LMTOFFSET);
}
