/*++ quereqvlpq.c - Network Queueing System
 *
 * $Source$
 *
 * DESCRIPTION:
 *
 *	Send a "queue request in a local queue via a local
 *	pipe queue" packet to the NQS daemon.
 *
 *
 *	Author:
 *	-------
 *	Brent A. Kingsbury, Sterling Software Incorporated.
 *	December 18, 1985.
 *
 *
 * STANDARDS VIOLATIONS:
 *   None.
 *
 * REVISION HISTORY: ($Revision$ $Date$ $State$)
 * $Log$
 *
 */

#include NMAP				/* uid_t and mid_t */
#include "nqspacket.h"			/* NQS local message packets */

extern long inter();			/* Send local packet */
extern void interclear();		/* Clear packet */
extern void interw32i();		/* Write packet integer datum */
extern void interwstr();		/* Write packet string datum */

/*** quereqvlpq
 *
 *
 *	long quereqvlpq():
 *
 *	Send a "queue request in a local queue via a local
 *	pipe queue" packet to the NQS daemon.
 *
 *	Returns:
 *		TCML_COMPLETE:	 if the specified request was
 *				 successfully queued in the local
 *				 destination queue.
 *		TCML_INTERNERR:	 if maximum packet size exceeded.
 *		TCML_NOESTABLSH: if unable to establish inter-
 *				 process communication.
 *		TCML_NOLOCALDAE: if the NQS daemon is not running.
 *		TCML_NOSUCHQUE:	 if the specified destination
 *				 queue does not exist.
 *		TCML_NOSUCHREQ:	 if the specified request does not
 *				 exist.
 *		TCML_PROTOFAIL:	 if a protocol failure occurred.
 */
long quereqvlpq (orig_seqno, orig_mid, destqueue)
long orig_seqno;			/* Original sequence# */
mid_t orig_mid;				/* Original machine-id of req */
char *destqueue;			/* Name of local queue where */
{					/* request should be queued */
	interclear();
	interw32i ((long) orig_seqno);
	interw32i ((long) orig_mid);
	interwstr (destqueue);
	return (inter (PKT_QUEREQVLPQ));
}
