/*++ rcimsgs.c - Network Queueing System
 *
 * $Source$
 *
 * DESCRIPTION:
 *
 *
 *	Diagnose request code information bits.
 *
 *
 *	Author:
 *	-------
 *	Brent A. Kingsbury, Sterling Software Incorporated.
 *	April 28, 1986.
 *
 *
 * STANDARDS VIOLATIONS:
 *   None.
 *
 * REVISION HISTORY: ($Revision$ $Date$ $State$)
 * $Log$
 *
 */

#include <stdio.h>		/* FILE definition */
#include "informcc.h"		/* NQS information completion codes */

/*** rcimsgs
 *
 *
 *	void rcimsgs():
 *	Diagnose request code information bits.
 */
void rcimsgs (code, stream, prefix, headerfn)
register long code;			/* Completion code */
register FILE *stream;			/* Output stream */
register char *prefix;			/* Output prefix */
register void (*headerfn)();		/* Header display function */
{
	register short preamble;	/* Preamble seen flag */

	preamble = -1;			/* No preamble yet */
	if (code & RCI_ACCESSDEN) {
		preamble = 0;
		(*headerfn)(stream, prefix);
		fputs (prefix, stream);
		fputs ("  Access denied;\n", stream);
	}
	if (code & RCI_CLIMIDUNKN) {
		if (!(++preamble)) (*headerfn)(stream, prefix);
		fputs (prefix, stream);
		fputs ("  Client machine-id is unknown at transaction peer;\n",
		       stream);
	}
	if (code & RCI_EFBIG) {
		if (!(++preamble)) (*headerfn)(stream, prefix);
		fputs (prefix, stream);
		fputs ("  File size limit exceeded;\n", stream);
	}
	if (code & RCI_FATALABORT) {
		if (!(++preamble)) (*headerfn)(stream, prefix);
		fputs (prefix, stream);
		fputs ("  Non-recoverable transaction failure;\n", stream);
	}
	if (code & RCI_MIDCONFLICT) {
		if (!(++preamble)) (*headerfn)(stream, prefix);
		fputs (prefix, stream);
		fputs (
		  "  Machine-id conflict between client and destination;\n",
		     stream);
	}
	if (code & RCI_NETNOTSUPP) {
		if (!(++preamble)) (*headerfn)(stream, prefix);
		fputs (prefix, stream);
		fputs ("  Networking not supported at NQS site.\n", stream);
	}
	if (code & RCI_NETPASSWD) {
		if (!(++preamble)) (*headerfn)(stream, prefix);
		fputs (prefix, stream);
		fputs ("  Network password verification error;\n", stream);
	}
	fflush (stream);		/* For logfile */
	if (code & RCI_NOSUCHFORM) {
		if (!(++preamble)) (*headerfn)(stream, prefix);
		fputs (prefix, stream);
		fputs ("  No such device forms;\n", stream);
	}
	if (code & RCI_NOSUCHQUE) {
		if (!(++preamble)) (*headerfn)(stream, prefix);
		fputs (prefix, stream);
		fputs ("  No such queue;\n", stream);
	}
	if (code & RCI_PEERINTERR) {
		if (!(++preamble)) (*headerfn)(stream, prefix);
		fputs (prefix, stream);
		fputs ("  NQS internal error at transaction peer;\n", stream);
	}
	if (code & RCI_PEERMIDUNKN) {
		if (!(++preamble)) (*headerfn)(stream, prefix);
		fputs (prefix, stream);
		fputs ("  Local machine-id is unknown at transaction peer;\n",
		       stream);
	}
	if (code & RCI_PEERNETDB) {
		if (!(++preamble)) (*headerfn)(stream, prefix);
		fputs (prefix, stream);
		fputs ("  Network database error at transaction peer;\n",
		       stream);
	}
	if (code & RCI_PEERNOACATH) {
		if (!(++preamble)) (*headerfn)(stream, prefix);
		fputs (prefix, stream);
		fputs ("  No account authorization at transaction peer;\n",
		       stream);
	}
	fflush (stream);		/* For logfile */
	if (code & RCI_PROTOFAIL) {
		if (!(++preamble)) (*headerfn)(stream, prefix);
		fputs (prefix, stream);
		fputs ("  NQS protocol failure;\n", stream);
	}
	if (code & RCI_QUOTALIMIT) {
		if (!(++preamble)) (*headerfn)(stream, prefix);
		fputs (prefix, stream);
		fputs ("  Explicit request quota limits exceed maximums;\n",
		       stream);
	}
	if (code & RCI_RRFUNKNMID) {
		if (!(++preamble)) (*headerfn)(stream, prefix);
		fputs (prefix, stream);
		fputs (
		  "  Request refers to mids unknown at transaction peer;\n",
		   stream);
	}
	if (code & RCI_WROQUETYP) {
		if (!(++preamble)) (*headerfn)(stream, prefix);
		fputs (prefix, stream);
		fputs ("  Wrong queue type for request;\n", stream);
	}
	if (code & RCI_UNAFAILURE) {
		if (!(++preamble)) (*headerfn)(stream, prefix);
		fputs (prefix, stream);
		fputs ("  Unanticipated transaction failure;\n", stream);
	}
	fflush (stream);		/* For logfile */
}
