/*++ verifyhdr.c - Network Queueing System
 *
 * $Source$
 *
 * DESCRIPTION:
 *
 *	Verify the common portion of a rawreq structure.
 *
 *
 *	Author:
 *	-------
 *	Brent A. Kingsbury, Sterling Software Incorporated.
 *	March 28, 1986.
 *
 *
 * STANDARDS VIOLATIONS:
 *   None.
 *
 * REVISION HISTORY: ($Revision$ $Date$ $State$)
 * $Log$
 *
 */

#include "nqs.h"


/*** verifyhdr
 *
 *
 *	int verifyhdr():
 *	Verify the common portion of a rawreq structure.
 *
 *	Returns:
 *		 0: if common header is valid.
 *		-1: otherwise.
 */
int verifyhdr (rawreq)
struct rawreq *rawreq;			/* Raw request header */
{
	/*
	 * "rawreq->ndatafiles > 32767" will be an invariant comparison
	 * when compiled on a machine with 16 bit shorts.
	 */
	if (rawreq->magic1 != REQ_MAGIC1 ||
	    rawreq->trans_id < 0 || rawreq->trans_id > MAX_TRANSACTS ||
	    rawreq->trans_quename [MAX_QUEUENAME] != '\0' ||
	    rawreq->quename [0] == '\0' ||
	    rawreq->quename [MAX_QUEUENAME] != '\0' ||
	    rawreq->type != RTYPE_BATCH && rawreq->type != RTYPE_DEVICE ||
	    rawreq->orig_seqno < -1 || rawreq->orig_seqno > MAX_SEQNO_USER ||
	    rawreq->rpriority < -1 || rawreq->rpriority > MAX_RPRIORITY ||
	    rawreq->ndatafiles < 0 || rawreq->ndatafiles > 32767 ||
	    rawreq->reqname [0] == '\0' ||
	    rawreq->reqname [MAX_REQNAME] != '\0' ||
	    rawreq->username [0] == '\0' ||
	    rawreq->username [MAX_ACCOUNTNAME] != '\0' ||
	    rawreq->mail_name [0] == '\0' ||
	    rawreq->mail_name [MAX_ACCOUNTNAME] != '\0') {
		/*
		 *  Values out of bounds.
		 */
		return (-1);
	}
	/*
	 *  No errors detected in the common portion of the rawreq.
	 */
	return (0);
}
