/*++ dir.4.2.h - 
 *
 * $Source$
 *
 * DESCRIPTION:
 *
 *	Emulate the Berkeley UNIX directory structure on top
 *	of the System V fixed name length directory structure.
 *
 *	The following comments describe the structure of a Berkeley
 *	UNIX directory:
 *
 *	A directory consists of some number of blocks of DIRBLKSIZ
 *	bytes, where DIRBLKSIZ is chosen such that it can be transferred
 *	to disk in a single atomic operation (e.g. 512 bytes on most machines).
 *
 *	Each DIRBLKSIZ byte block contains some number of directory entry
 *	structures, which are of variable length.  Each directory entry has
 *	a struct direct at the front of it, containing its inode number,
 *	the length of the entry, and the length of the name contained in
 *	the entry.  These are followed by the name padded to a 4 byte boundary
 *	with null bytes.  All names are guaranteed null terminated.
 *	The maximum length of a name in a directory is MAXNAMLEN.
 *
 *	The macro DIRSIZ(dp) gives the amount of space required to represent
 *	a directory entry.  Free space in a directory is represented by
 *	entries which have dp->d_reclen >= DIRSIZ(dp).  All DIRBLKSIZ bytes
 *	in a directory block are claimed by the directory entries.  This
 *	usually results in the last entry in a directory having a large
 *	dp->d_reclen.  When entries are deleted from a directory, the
 *	space is returned to the previous entry in the same directory
 *	block by increasing its dp->d_reclen.  If the first entry of
 *	a directory block is free, then its dp->d_ino is set to 0.
 *	Entries other than the first in a directory do not normally have
 *	dp->d_ino set to 0.
 *
 *
 *	Author:
 *	-------
 *	Kirk McKusick, University of California, Berkeley.
 *	March 10, 1982.
 *
 *	Modifications:
 *	--------------
 *	Brent A. Kingsbury, Sterling Software Incorporated.
 *	September 10, 1985.
 *
 *	  Changed "#include <dir.h>" to read '#include "dir.4.2.h"' since
 *	  we are after the Berkeley definition, and not System V.
 *
 *
 * STANDARDS VIOLATIONS:
 *
 * REVISION HISTORY: ($Revision$ $Date$ $State$)
 * $Log$
 *
 */

/*
 * 
 *	Define the directory block allocation size in bytes.
 *
 *	WARNING:
 *	  Seven system types are defined:  BSD42, BSD43, SGI, SYS52, SYS53,
 *	  ULTRIX, UNICOS, and UTS.  Accordingly, the flag: SYSTEM_TYPE
 *	  in the Makefile must be appropriately defined.
 *
 *
 */
#if	BSD42 | BSD43 | ULTRIX		/* Berkeley UNIX */
#define	DIRBLKSIZ	512
#else
#ifdef	SGI				/* Silicon Graphics Iris UNIX */
#define	DIRBLKSIZ	512
#else
#ifdef	SYS52 | SYS53				/* AT&T UNIX */
#define	DIRBLKSIZ	512
#else
#ifdef	UTS				/* Amdahl Unix */
#define	DIRBLKSIZ	4096
#else
#ifdef	UNICOS				/* Cray UNIX */
#define	DIRBLKSIZ	4096
#else
BAD SYSTEM TYPE
#endif
#endif
#endif
#endif
#endif
#define	MAXNAMLEN	255

struct	direct {
#if	BSD42 | BSD43 | ULTRIX | SGI | SYS52 | SYS53 | UTS
	unsigned long d_ino;		/* inode number of entry */
#else
#if	UNICOS
	unsigned int d_ino;		/* inode number of entry */
	unsigned int d_dev;		/* Device number of entry */
#else
BAD SYSTEM TYPE
#endif
#endif
	unsigned short d_reclen;	/* length of this record */
	unsigned short d_namlen;	/* length of string in d_name */
	char	d_name[MAXNAMLEN + 1];	/* name must be no longer than this */
};

/*
 * The DIRSIZ macro gives the minimum record length which will hold
 * the directory entry.  This requires the amount of space in struct direct
 * without the d_name field, plus enough space for the name with a terminating
 * null byte (dp->d_namlen+1), rounded up to a 4 byte boundary.
 */
#undef DIRSIZ
#define DIRSIZ(dp) \
    ((sizeof (struct direct) - (MAXNAMLEN+1)) + (((dp)->d_namlen+1 + 3) &~ 3))

#ifndef KERNEL
/*
 * Definitions for library routines operating on directories.
 */
typedef struct _dirdesc {
	int	dd_fd;
	long	dd_loc;
	long	dd_size;
	char	dd_buf[DIRBLKSIZ];
} DIR;
#ifndef NULL
#define NULL 0
#endif
extern	DIR *opendir();
extern	struct direct *readdir();
extern	long telldir();
extern	void seekdir();
#define rewinddir(dirp)	seekdir((dirp), (long)0)
extern	void closedir();
#endif KERNEL
