/*++ readdir.c - 
 *
 * $Source$
 *
 * DESCRIPTION:
 *
 *	Emulate the Berkeley 4.2 UNIX library call of:  readdir (3x)
 *	On top of the System V fixed name length directory structure.
 *
 *
 *	Author:
 *	-------
 *	Kirk McKusick, University of California, Berkeley.
 *	March 10, 1982.
 *
 *	Modifications:
 *	--------------
 *	Brent A. Kingsbury, Sterling Software Incorporated.
 *	September 10, 1985.
 *
 *	  Changed "#include <dir.h>" to read "#include "dir.4.2.h"" since
 *	  we are after the Berkeley definition, and not System V.  Also
 *	  changed olddirect for UNICOS.
 *
 *
 * STANDARDS VIOLATIONS:
 *
 * REVISION HISTORY: ($Revision$ $Date$ $State$)
 * $Log$
 *
 */

#include <sys/types.h>
#if	SGI | SYS52 | SYS53 | UNICOS | UTS
#include "dir.4.2.h"
#else
#if	BSD42 | BSD43 | ULTRIX
#include <sys/dir.h>
#else
BAD SYSTEM TYPE
#endif
#endif

/*
 * read an old style directory entry and present it as a new one
 */
#define	ODIRSIZ	14

#if	SGI | SYS52 | SYS53 | UTS
struct	olddirect {
	ino_t	od_ino;
	char	od_name[ODIRSIZ];
};
#else
#if	UNICOS
struct	olddirect {
	dino_t	od_ino;		/* Primary i-node; od_dino.ino = 0 implies */
				/* that this directory entry is not in use */
	dino_t	od_shadow;	/* A redundant i-node */
	char	od_name [ODIRSIZ];
};
#else
BAD SYSTEM TYPE
#endif
#endif

/*** readdir
 *
 *
 *	struct direct *readdir():
 *	Read the next entry in a directory.
 */
struct direct *readdir(dirp)
register DIR *dirp;
{
	register struct olddirect *dp;
	static struct direct dir;

	for (;;) {
		if (dirp->dd_loc == 0) {
			dirp->dd_size = read(dirp->dd_fd, dirp->dd_buf, 
			    DIRBLKSIZ);
			if (dirp->dd_size <= 0) return (NULL);
		}
		if (dirp->dd_loc >= dirp->dd_size) {
			dirp->dd_loc = 0;
			continue;
		}
		dp = (struct olddirect *)(dirp->dd_buf + dirp->dd_loc);
		dirp->dd_loc += sizeof(struct olddirect);
#if	SGI | SYS52 | SYS53 | UTS
		if (dp->od_ino == 0)
#else
#if	UNICOS
		if (dp->od_ino.ino == 0)
#else
BAD SYSTEM TYPE
#endif
#endif
			continue;
#if	BSD42 | BSD43 | ULTRIX | SGI | SYS52 | SYS53 | UTS
		dir.d_ino = dp->od_ino;
#else
#if	UNICOS
		dir.d_ino = dp->od_ino.ino;
		dir.d_dev = dp->od_ino.dev;
#else
BAD SYSTEM TYPE
#endif
#endif
		strncpy(dir.d_name, dp->od_name, ODIRSIZ);
		dir.d_name[ODIRSIZ] = '\0'; /* insure null termination */
		dir.d_namlen = strlen(dir.d_name);
		dir.d_reclen = DIRBLKSIZ;
		return (&dir);
	}
}
