/*++ gethbynam.c - Network Queueing System
 *
 * $Source$
 *
 * DESCRIPTION:
 *
 *	Emulate a Berkeley 4.2 UNIX style (and NPSN UNIX style)
 *	gethostbyname() call for UNIX systems not supporting this
 *	call.  See NPSN_EMULATE in ../proto/Makefile.
 *
 *
 *	Author:
 *	-------
 *	Brent A. Kingsbury, Sterling Software Incorporated.
 *	August 12, 1985.
 *
 *
 * STANDARDS VIOLATIONS:
 *   None.
 *
 * REVISION HISTORY: ($Revision$ $Date$ $State$)
 * $Log$
 *
 */

#include NETDB			/* Network database */
extern char *strncpy();		/* Copy n characters */

/*** gethostbyname
 *
 *
 *	struct hostent *gethostbyname():
 *
 *	Return an emulated hostent structure as would have been
 *	returned from a genuine Berkeley 4.2 or NPSN UNIX system
 *	gethostbyname() function call.
 *
 *	Returns:
 *		A pointer to the returned hostent structure.
 */
struct hostent *gethostbyname (name)
char *name;
{
	static char savedname [256];
	static struct hostent hostent;
	static char *aliases[1] = {
		(char *) 0
	};

	strncpy (savedname, name, 255);	/* Save name given by caller */
	savedname [255] = '\0';		/* Make sure it is null terminated */
	hostent.h_name = savedname;	/* Place name in the hostent struct */
	hostent.h_aliases = aliases;	/* Place empty aliases list ... */
	hostent.h_addrtype = 0;		/* In real life, this is usually */
					/* AF_INET */
	hostent.h_length = 0;		/* Length of address is 0 */
	hostent.h_addr = aliases [0];	/* No address */
	return (&hostent);		/* Return hostent structure */
}
