/*++ gethostnam.c - Network Queueing System
 *
 * $Source$
 *
 * DESCRIPTION:
 *
 *	Emulate a Berkeley 4.2 UNIX style (and NPSN UNIX style)
 *	gethostname() call for UNIX systems not supporting this
 *	call.  See NPSN_EMULATE in ../proto/Makefile.
 *
 *
 *	Author:
 *	-------
 *	Brent A. Kingsbury, Sterling Software Incorporated.
 *	August 12, 1985.
 *
 *
 * STANDARDS VIOLATIONS:
 *   None.
 *
 * REVISION HISTORY: ($Revision$ $Date$ $State$)
 * $Log$
 *
 */

#include <fcntl.h>
extern char *strncpy();		/* Copy n characters */

/*** gethostname
 *
 *
 *	int gethostname():
 *
 *	Get the name of the local host system as stored in the
 *	file: /usr/lib/nqs/sitename.
 *
 *	Note that the name will NOT be null terminated and will
 *	also be truncated as necessary if the specified buffer
 *	is too small.
 *
 *	Returns:
 *		0: This emulation of the real function always
 *		   returns 0.
 */
int gethostname (namespace, namelen)
char *namespace;			/* Address of buffer to hold name */
int namelen;				/* Size of buffer in characters */
{
	static char *localhost = "localhost";

	char buffer [256];
	int n_read;
	int fd;
	register char *scan;
	register int length;

	if ((fd = open ("/usr/lib/nqs/sitename", O_RDONLY)) == -1) {
		strncpy (namespace, localhost, namelen);
	}
	else {
		if ((n_read = read (fd, buffer, 255)) <= 0) {
			strncpy (namespace, localhost, namelen);
		}
		else {
			buffer [n_read] = '\0';	/* Null terminate */
			scan = buffer;
			length = 0;
			while (*scan && *scan != '\n') {
				scan++;
				length++;
			}
			buffer [length] = '\0';	/* Stamp-out '\n' */
			if (length == 0) {
				strncpy (namespace, localhost, namelen);
			}
			else strncpy (namespace, buffer, namelen);
		}
		close (fd);	/* Close the sitename file */
	}
	return (0);		/* Always return 0 */
}
