/*++ ioblksiz.h - optimal block size for indivisible file system I/O operations
 *
 * $Source$
 *
 * DESCRIPTION:
 *
 *	This header file defines the optimal block size for indivisible
 *	file system I/O operations.  This value defines the largest
 *	number of bytes that can be read/written from/to a file in a
 *	single indivisible I/O operation (ATOMICBLKSIZ).  It is also
 *	permissible for this value to be an exact divisor of the true
 *	"atomic block size."  In either case, this value MUST satisfy
 *	these constraints for the supporting system.
 *
 *
 *	WARNING:
 *	  Seven system types are defined:  BSD42, BSD43, SGI, SYS52, SYS53,
 *	  ULTRIX, UNICOS, and UTS.  Accordingly, the flag: SYSTEM_TYPE
 *	  in the Makefile must be appropriately defined.
 *
 *	  The definition of ATOMICBLKSIZ must NOT be changed without
 *	  careful forethought.  If the value of ATOMICBLKSIZ is changed,
 *	  then the database software that is dependent upon the correct
 *	  definition of ATOMICBLKSIZ can be easily broken.  Examples of
 *	  such software include the nmap (network mapping) database
 *	  routines, and the Network Queueing System (NQS).
 *
 *	  Be careful.
 *
 *
 *	Author(s):
 *	----------
 *	Brent A. Kingsbury, and Kate R. Rosenbloom.
 *	Sterling Software Incorporated.
 *
 *
 * STANDARDS VIOLATIONS:
 *
 * REVISION HISTORY: ($Revision$ $Date$ $State$)
 * $Log$
 *
 */

#if	IO512
#define ATOMICBLKSIZ	512		/* Old style */
#else

#if	BSD42 | BSD43 | ULTRIX		/* Berkeley UNIX */
#define	ATOMICBLKSIZ	4096		/* (On some Berkeley systems, it */
					/* might be 8192) */
#else
#if	SGI				/* Silicon Graphics Iris UNIX */
#define	ATOMICBLKSIZ	1024
#else
#if	SYS52 | SYS53				/* AT&T garden variety UNIX */
#define	ATOMICBLKSIZ	1024
#else
#if	UTS				/* Amdahl Unix */
#define	ATOMICBLKSIZ	4096
#else
#if	UNICOS				/* Cray UNIX */
#define	ATOMICBLKSIZ	4096
#else
BAD SYSTEM TYPE
#endif
#endif
#endif
#endif
#endif
#endif
