/*++ nmapread.c -
 *
 * $Source$
 *
 * DESCRIPTION:
 *	Nmapread.c:  Routines to read nmap database
 *
 * STANDARDS VIOLATIONS:
 *
 * REVISION HISTORY: ($Revision$ $Date$ $State$)
 * $Log$
 *
 *	May 6, 1991	Add show capability
 *
 */

#include "nmapcommon.h"


/*
 *
 *
 *	int nmap_get_uid (from_mid, from_uid, &local_uid)
 *
 *	Map the user-id: from_uid from the machine with mid: from_mid
 *	to the local user-id: local_uid.
 *
 *	Returns:
 *	   0: (NMAP_SUCCESS) if successful.
 *	   1: (NMAP_DEFMAP) if no mapping existed for the specified
 *	      machine and user-id, but a default user-id mapping
 *	      was enabled for the specified machine.
 *	  -1: (NMAP_EUNEXPECT) if an unanticipated error occurred in
 *	      the mapping software.
 *	  -2: (NMAP_ENOPRIV) if the caller did not have sufficient
 *	      privilege to access the mapping database.
 *	  -4: (NMAP_ENOMAP) if no mapping existed for the specified
 *	      machine and user-id.
 */
int nmap_get_uid (from_mid, from_uid, local_uid)
mid_t from_mid;
uid_t from_uid;
uid_t *local_uid;
{
	struct xidparm parm;
	register int res;
 
	parm.u.u.remote_uid = from_uid;
	res = xidop (from_mid, &parm, GET_UID);
	if (res >= 0) *local_uid = parm.u.u.local_uid;
	nmap_ctl (NMAP_OPNOFD, nfds);
			/* Relinquish file descriptors if necessary */
	return (res);
}


/*
 *
 *
 *	int nmap_get_gid (from_mid, from_gid, &local_gid)
 *
 *	Map the user-id: from_gid from the machine with mid: from_mid
 *	to the local user-id: local_gid.
 *
 *	Returns:
 *	   0: (NMAP_SUCCESS) if successful.
 *	   1: (NMAP_DEFMAP) if no mapping existed for the specified
 *	      machine and group-id, but a default group-id mapping
 *	      was enabled for the specified machine.
 *	  -1: (NMAP_EUNEXPECT) if an unanticipated error occurred in
 *	      the mapping software.
 *	  -2: (NMAP_ENOPRIV) if the caller did not have sufficient
 *	      privilege to access the mapping database.
 *	  -4: (NMAP_ENOMAP) if no mapping existed for the specified
 *	      machine and group-id.
 */
int nmap_get_gid (from_mid, from_gid, local_gid)
mid_t from_mid;
gid_t from_gid;
gid_t *local_gid;
{
	struct xidparm parm;
	register int res;
 
	parm.u.g.remote_gid = from_gid;
	res = xidop (from_mid, &parm, GET_GID);
	if (res >= 0) *local_gid = parm.u.g.local_gid;
	nmap_ctl (NMAP_OPNOFD, nfds);
			/* Relinquish file descriptors if necessary */
	return (res);
}


/*
 *
 *
 *	int nmap_get_mid (&hostent, &mid)
 *
 *	Determine the machine-id (mid) of the system identified by
 *	the specified hostent structure.
 *
 *	NOTE:	If the passed address of the hostent structure is NULL,
 *		then the machine-id the local machine upon which the
 *		calling process is running is determined by this
 *		function call.
 *
 *	Returns:
 *	   0: (NMAP_SUCCESS) if successful.
 *	  -1: (NMAP_EUNEXPECT) if an unanticipated error occurred in
 *	      the mapping software.
 *	  -2: (NMAP_ENOPRIV) if the caller did not have sufficient
 *	      privilege to access the mapping database.
 *	  -4: (NMAP_ENOMAP) if no mapping existed for the specified
 *	      hostent structure.
 */
int nmap_get_mid (hostent, mid)
struct hostent *hostent;
mid_t *mid;
{
	extern int gethostname();		/* Get local host name */
	extern struct hostent *gethostbyname();	/* Get host by name */

	char hostname [MAX_NAMELENGTH+1];	/* Local host name */
	register int status;
	register int alias;

	if (hostent == (struct hostent *) 0) {
		/*
		 *  No hostent structure was specified.  The caller
		 *  is asking for the machine-id of the local machine.
		 */
		if (gethostname (hostname, MAX_NAMELENGTH) == -1) {
			/*
			 *  We were unable to determine the name of the
			 *  local machine!
			 */
			return (NMAP_EUNEXPECT);/* We were surprised */
		}
		hostname [MAX_NAMELENGTH]='\0';	/* Null terminate for sure */
		if ((hostent = gethostbyname(hostname))==(struct hostent *)0) {
			return(NMAP_EUNEXPECT);	/* Surprised again! */
		}
	}
	alias = 0;				/* First alias in list */
	status = namop (hostent->h_name, mid, GET_MID);
	if (hostent->h_aliases != (char **)0) {
		while (status == NMAP_ENOMAP && hostent->h_aliases[alias] != (char *)0){
			/*
			 *  While no mapping, check aliases for a mapping
			 *  until we run out of aliases.
			 */
			status = namop (hostent->h_aliases [alias++], mid, GET_MID);
		}
	}
	nmap_ctl (NMAP_OPNOFD, nfds);
			/* Relinquish file descriptors if necessary */
	return (status);	/* Return result */
}


/*
 *
 *
 *	struct hostent *nmap_get_ent (mid):
 *	Return a hostent structure for the specified machine-id.
 *
 *	Returns:
 *		A pointer to a hostent structure for the specified
 *		machine-id if successful; otherwise NULL is returned.
 */
struct hostent *nmap_get_ent (mid)
mid_t mid;			/* Where to return the machine-id */
{
	extern struct hostent *gethostbyname();	/* Get host by name */
	char *nmap_get_nam();	/* Get principal name for mid */

	char *hostname;		/* Principal name for machine-id */

	if ((hostname = nmap_get_nam (mid)) == (char *) 0) {
		return ((struct hostent *) 0);
	}
	return (gethostbyname (hostname));
}


/*
 *
 *
 *	char *nmap_get_nam (mid)
 *
 *	Return a pointer to the null terminated character string
 *	containing the principal host name for the specified machine-id.
 *	Return NULL for all error conditions (including no mapping for
 *	"mid").
 *
 *	WARNING:  The name is placed in a static data area which is
 *		  overwritten with each call.
 */
char *nmap_get_nam (mid)
mid_t mid;
{

	static char name [MAX_NAMELENGTH+1];	/* Name space */
	struct rawblock block;
	register int i;

	if (open_machine (mid, O_RDONLY) != NMAP_SUCCESS) {
		nmap_ctl (NMAP_OPNOFD, nfds);
			/* Relinquish file descriptors if necessary */
		return ((char *) 0);	/* NMAP_EUNEXPECT or NMAP_ENOPRIV */
	}
	/*
	 *  Seek to beginning of file if not already there.
	 */
	if (LSEEK (machine_file, 0L, 0) == -1) {
		nmap_ctl (NMAP_OPNOFD, nfds);
			/* Relinquish file descriptors if necessary */
		return ((char *) 0);	/* NMAP_EUNEXPECT */
	}
	/*
	 *  Get the principal name of the machine from the first
	 *  block of the uid/gid mapping file for the target machine.
	 */
	if (READ (machine_file, block.u.bytes, ATOMICBLKSIZ) != ATOMICBLKSIZ) {
		nmap_ctl (NMAP_OPNOFD, nfds);
			/* Relinquish file descriptors if necessary */
		return ((char *) 0);
	}
	i = 0;
	while (block.u.mhdr.name [i] != '\0' && i < MAX_NAMELENGTH) {
		name [i] = block.u.mhdr.name [i];
		i++;
	}
	name [i] = '\0';	/* Null terminate */
	nmap_ctl (NMAP_OPNOFD, nfds);
			/* Relinquish file descriptors if necessary */
	return (name);
}


/*
 *
 *
 *	int nmap_get_defuid (from_mid, &defuid)
 *
 *
 *	Get the default user-id mapping for the machine with
 *	mid: from_mid to the default user-id: defuid.
 *
 *	Returns:
 *	   0: (NMAP_SUCCESS) if successful.
 *	  -1: (NMAP_EUNEXPECT) if an unanticipated error occurred in
 *	      the mapping software.
 *	  -2: (NMAP_ENOPRIV) if the caller did not have sufficient
 *	      privilege to update the mapping database.
 *	  -4: (NMAP_ENOMAP) if default uid doesn't exist for this mid
 *	  -5: (NMAP_ENOMID) if no such machine as specified by
 *	      the machine-id (mid) exists.
 */
int nmap_get_defuid (from_mid, defuid)
mid_t from_mid;
uid_t *defuid;
{

	struct rawblock block;
	int status;

	if ((status = open_machine (from_mid, O_RDONLY)) != NMAP_SUCCESS) {
		nmap_ctl (NMAP_OPNOFD, nfds);
			/* Relinquish file descriptors if necessary */
		return (status);   /* NMAP_EUNEXPECT, NMAP_ENOMID, or */
				   /* NMAP_ENOPRIV */
	}
	/*
	 *  Seek to beginning of file if not already there.
	 */
	if (LSEEK (machine_file, 0L, 0) == -1) {
		nmap_ctl (NMAP_OPNOFD, nfds);
			/* Relinquish file descriptors if necessary */
		return (NMAP_EUNEXPECT);
	}
	if (READ (machine_file, block.u.bytes, ATOMICBLKSIZ) != ATOMICBLKSIZ) {
		nmap_ctl (NMAP_OPNOFD, nfds);
			/* Relinquish file descriptors if necessary */
		return (NMAP_EUNEXPECT);
	}
	if ( block.u.mhdr.defuid_enabled !=  1 )
		return (NMAP_ENOMAP);

	*defuid = block.u.mhdr.defuid;		/* Store default */
	nmap_ctl (NMAP_OPNOFD, nfds);
			/* Relinquish file descriptors if necessary */
	return (NMAP_SUCCESS);	/* Return success */
}


/*
 *
 *
 *	int nmap_sho_mid ()
 *
 *	Display the mids, principal names, and aliases for all of the
 *	machines in the database. Also indicate any potential inconsistencies
 *	in the database (e.g. no mid file, principal name not in machines file).
 *
 *	Returns:
 * 	 0: (NMAP_SUCCESS)	if successful.
 *	 -1: (NMAP_EUNEXPECT)	major error.
 *	 -2: (NMAP_ENOPRIV)	denied access privilege.
 *	 -11: (NMAP_ENOMAPFILE)	The mapping file does not exist.
 *
 */
int nmap_sho_mid ()
{
	int nummids = 0;		/* number of mids read from database */
	mid_t mids[MAX_MIDS+1];		/* list of mids read from database */
	mid_t testmid;
	char *pname;			/* pointer to principal name */
	char *pname_str;		/* pointer to principal name not */
					/* found string */
	short not_found;		/* principal name not found flag */
	register int status;		/* Status return */
	register int j;			/* index */
	int nmap_scan_map();
	int nmap_sort_mids();
	int nmap_print_mids();

	/*
	 * initialize
	 */
	not_found = 0;
	for (j = 0; j <= MAX_MIDS; j++) mids[j] = 0x7fffffff;

	if ((status = open_name_map (O_RDONLY)) != NMAP_SUCCESS) {
		nmap_ctl (NMAP_OPNOFD, nfds);
			/* Relinquish file descriptors if necessary */
		return (status);	/* Return -1: Fatal error */
					/*	  -2: Insufficient privilege */
	}
	/*
	 * collect all of the currently defined mids in sorted order
	 */
	if ((status = nmap_scan_map (nmap_sort_mids, (char *)mids,
					(char *)&nummids)) != NMAP_SUCCESS) {
		nmap_ctl (NMAP_OPNOFD, nfds);
			/* Relinquish file descriptors if necessary */
		return (status);
	}
	if (nummids > MAX_MIDS) {
		printf("********** Encountered more mids in the database than\
 MAX_MIDS\n");
		printf("********** Will process only MAX_MIDS = %d\n\n",
 MAX_MIDS);
		nummids = MAX_MIDS;
	}
	/*
	 *      print header
	 */
	printf (" MID\tPRINCIPAL NAME\tALIASES\n");
	printf (" ---\t--------------\t-------");
	/*
	 *      print all mids, principal names, and aliases
	 */
	for (j = 0; j < nummids; j++) {
		pname_str = "";
		if ((pname = nmap_get_nam (mids[j])) == (char *) 0)
			pname = "????";
		else if (namop (pname, &testmid, GET_MID) != NMAP_SUCCESS ||
			testmid != mids[j]) {
				pname_str = "**";
				not_found = 1;
		}
		printf("\n%4ld%4s%-15s\t", mids[j], pname_str, pname);
		/*
		 *   Print aliases
		 */
		if ((status = open_name_map (O_RDONLY)) != NMAP_SUCCESS) {
			nmap_ctl (NMAP_OPNOFD, nfds);
				/* Relinquish file descriptors if necessary */
			return (status);	/* Return -1: Fatal error */
		}
		if ((status = nmap_scan_map (nmap_print_mids, (char *)mids[j],
						pname)) != NMAP_SUCCESS) {
			nmap_ctl (NMAP_OPNOFD, nfds);
			/* Relinquish file descriptors if necessary */
			return (status);
		}
	}
	printf ("\n");
	if (not_found)
		printf ("\t** - Principal name not in machines file\n");
	nmap_ctl (NMAP_OPNOFD, nfds);
		/* Relinquish file descriptors if necessary */
	return (NMAP_SUCCESS);
}


/*
 *
 *
 *	int nmap_sho_uid ()
 *
 *	Display the mid, principal name, and users mapping for all of the
 *	users in the database.
 *
 *	Returns:
 * 	 0: (NMAP_SUCCESS)	if successful.
 *	 -1: (NMAP_EUNEXPECT)	major error.
 *	 -2: (NMAP_ENOPRIV)	denied access privilege.
 *	 -11: (NMAP_ENOMAPFILE)	The mapping file does not exist.
 *
 */
int nmap_sho_uid (from_mid)
mid_t from_mid;
{
	mid_t testmid;
	char *pname;			/* pointer to principal name */
	char *pname_str;		/* pointer to principal name not */
					/* found string */
	register int status;		/* Status return */
	register int j;			/* index */
	uid_t local_uid;

	/*
	 *      print header
	 */
	printf (" MID\tPRINCIPAL NAME\tUID\tTOUID\n");
	printf (" ---\t--------------\t---\t-----\n");
	/*
	 *      find mid and principal name
	 */
	pname_str = "";
	if ((pname = nmap_get_nam (from_mid)) == (char *) 0)
		pname = "????";
	else if (namop (pname, &testmid, GET_MID) != NMAP_SUCCESS ||
		testmid != from_mid) {
			pname_str = "**";
	}
	/*
	 *   Print users mappings
	 */
	for (j = 0; j < 32767; j++) {
		if ((status = nmap_get_uid (from_mid, j, &local_uid))
			!= NMAP_SUCCESS) {
			nmap_ctl (NMAP_OPNOFD, nfds);
			/* Relinquish file descriptors if necessary */
		} else printf("%4ld%4s%-15s\t%d\t%d\n",
			from_mid, pname_str, pname,j, local_uid);
	}
	nmap_ctl (NMAP_OPNOFD, nfds);
		/* Relinquish file descriptors if necessary */
	return (NMAP_SUCCESS);
}


/*
 *
 *
 *	int nmap_sho_gid ()
 *
 *	Display the mid, principal name, and groups mappings for all of the
 *	groups in the database.
 *
 *	Returns:
 * 	 0: (NMAP_SUCCESS)	if successful.
 *	 -1: (NMAP_EUNEXPECT)	major error.
 *	 -2: (NMAP_ENOPRIV)	denied access privilege.
 *	 -11: (NMAP_ENOMAPFILE)	The mapping file does not exist.
 *
 */
int nmap_sho_gid (from_mid)
mid_t from_mid;
{
	mid_t testmid;
	char *pname;			/* pointer to principal name */
	char *pname_str;		/* pointer to principal name not */
					/* found string */
	register int status;		/* Status return */
	register int j;			/* index */
	gid_t local_gid;

	/*
	 *      print header
	 */
	printf (" MID\tPRINCIPAL NAME\tGID\tTOGID\n");
	printf (" ---\t--------------\t---\t-----\n");
	/*
	 *      find mid and principal name
	 */
	pname_str = "";
	if ((pname = nmap_get_nam (from_mid)) == (char *) 0)
		pname = "????";
	else if (namop (pname, &testmid, GET_MID) != NMAP_SUCCESS ||
		testmid != from_mid) {
			pname_str = "**";
	}
	/*
	 *   Print groups mappings
	 */
	for (j = 0; j < 32767; j++) {
		if ((status = nmap_get_gid (from_mid, j, &local_gid))
			!= NMAP_SUCCESS) {
			nmap_ctl (NMAP_OPNOFD, nfds);
			/* Relinquish file descriptors if necessary */
		} else printf("%4ld%4s%-15s\t%d\t%d\n",
			from_mid, pname_str, pname,j, local_gid);
	}
	nmap_ctl (NMAP_OPNOFD, nfds);
		/* Relinquish file descriptors if necessary */
	return (NMAP_SUCCESS);
}


/*
 *	int nmap_scan_map ()
 *
 *	Scan the machine database and call the passed function to
 *	process each entry
 *
 */
int nmap_scan_map (func, p1, p2)
	int (*func) ();		/* function to be called */
	char *p1, *p2;		/* this is a type violation and should be*/
				/* changed to (void *) when our compiler */
				/* gets around to supporting it */
{
	struct rawblock block;
	int status;		/* return status of func () */
	long address = HASHNAMBLKS * ATOMICBLKSIZ;
	unsigned offset;
	struct nammap *mmap;

	/*
	 *      Seek to first entry beyond hashblocks
	 */
	if (LSEEK (name_mapfile, address, 0) == -1) return (NMAP_EUNEXPECT);

	while ((status = READ (name_mapfile, block.u.bytes, ATOMICBLKSIZ))
									!= 0) {
		if (status == -1) return (NMAP_EUNEXPECT);
		/*
		 *  Skip over nextptr for hash chain link and
		 *  examine this block.
		 */
		offset = ((char *)(&block.u.nhdr.map))
		       - block.u.bytes;
		do {
			mmap = (struct nammap *)
			       (block.u.bytes + offset);
			if (mmap->namelen != 0) {
				if ((status = (*func) (mmap, p1, p2)) !=
					NMAP_SUCCESS) return(status);
			}
			/*
			 *  Prepare to access the next entry.
			 */
			offset += mmap->size;
		} while (offset < ATOMICBLKSIZ);
	}
	return(NMAP_SUCCESS);
}


/*
 *
 *	int nmap_sort_mids ()
 *
 *	Collect in sorted order each mid in the database
 *
 */
int nmap_sort_mids (mmap, mids, nummids)
	struct nammap *mmap;
	mid_t *mids;
	int *nummids;
{
	register int i;		/* index */
	register int j;		/* index */

	for (j = 0; j <= MAX_MIDS; j++) {
		if (mmap->mid == mids[j]) {
			return (NMAP_SUCCESS);
		}
		if (mmap->mid < mids[j]) {
			if ( ++(*nummids) > MAX_MIDS) return (NMAP_SUCCESS);
			/*
			 * move new mid into position
			 */
			for (i = *nummids; i > j; i--) {
				mids[i] = mids[i-1];
			}
			mids[j] = mmap->mid;
			return (NMAP_SUCCESS);
		}
	}
	return (NMAP_SUCCESS);
}


/*
 *
 *	int nmap_print_mids()
 *
 *	Print principal name and aliases per mid
 *
 */
int nmap_print_mids (mmap, mid, pname)
	struct nammap *mmap;
	mid_t mid;	/* current mid */
	char *pname;	/* pointer to principal name */
{
	char name[MAX_NAMELENGTH + 1];

	strncpy (name, mmap->name, mmap->namelen);
	name[mmap->namelen] = '\0';	/* null terminate for sure */
	if ((mid == mmap->mid) && (strcmp (pname, name) != 0)) {
		printf("%s ", name);
	}
	return(NMAP_SUCCESS);
}
