/*++ execsh.c - Network Queueing System
 *
 * $Source$
 *
 * DESCRIPTION:
 *
 *      Exec the requested shell process after the prologue and
 *      before the epilogue.
 *
 *
 *	Author:
 *	-------
 *      Jean-Philippe Baud, CERN, Geneva, Switzerland.
 *      January 13, 1988.
 *
 *
 * STANDARDS VIOLATIONS:
 *   None.
 *
 * REVISION HISTORY: ($Revision$ $Date$ $State$)
 * $Log$
 *
 *	May 13, 1991	Execution of the epilogue after receiving SIGTERM
 *	July 8, 1991	Generalization for most of the signals...
 *
 */

#include <stdio.h>
#include <signal.h>
#include "nqs.h"			/* NQS constants and data types */

/*
 *	External functions and variables:
 */

extern char *getenv();			/* Get environment var */
#if	SGI | SYS52 | SYS53 | UTS | UNICOS
extern char *strrchr();			/* Find last occurrence of char */
#else
#if	BSD42 | BSD43 | ULTRIX
extern char *rindex();			/* Find last occurrence of char */
#else
BAD SYSTEM TYPE
#endif
#endif

/*** main
 *
 *
 *      Invocation from the parent shell is as follows:
 *
 *        $(NQS_LIBEXE)/execsh
 *
 */
main (argc, argvp, envp)
int argc;
char *argvp[];
char *envp[];
{
	char *server;                   /* Pathname of batch shell */
	char *argv [MAX_SERVERARGS+1];	/* Execve() argument pointers */
	char minusname [257];		/* Name of shell (path removed) */
					/* prefixed with a "-" character */
	char *shellname;		/* Name of shell (path removed) */

	/*
	 *  Explicitly set the signal receipt actions to their defaults so
	 *  that the server gets a completely "clean" signal environment.
	 *  (Note that in most of the cases, except for SIGKILL and SIGSTOP,
	 *  the epilogue script is executed.)
	 */
	signal (SIGHUP, SIG_DFL);
	signal (SIGINT, SIG_DFL);
	signal (SIGQUIT, SIG_DFL);
	signal (SIGILL, SIG_DFL);
	signal (SIGTRAP, SIG_DFL);
#if	BSD42 | BSD43 | ULTRIX | SGI | SYS52 | SYS53 | UTS
	signal (SIGIOT, SIG_DFL);
	signal (SIGEMT, SIG_DFL);
#else
#if	UNICOS
	signal (SIGHWE, SIG_DFL);
#else
BAD SYSTEM TYPE
#endif
#endif
	signal (SIGFPE, SIG_DFL);
	signal (SIGSYS, SIG_DFL);
	signal (SIGPIPE, SIG_DFL);
	signal (SIGALRM, SIG_DFL);
	signal (SIGTERM, SIG_DFL);
#if	SYS52 | SYS53 | UTS
	signal (SIGUSR1, SIG_DFL);
	signal (SIGUSR2, SIG_DFL);
	signal (SIGCLD, SIG_DFL);
#ifndef	OSF1
	signal (SIGPWR, SIG_DFL);
#endif
#endif
#if	BSD42 | BSD43 | ULTRIX | SGI | SYS53
	signal (SIGURG, SIG_DFL);
	signal (SIGTSTP, SIG_DFL);
	signal (SIGCONT, SIG_DFL);
	signal (SIGCHLD, SIG_DFL);
	signal (SIGTTIN, SIG_DFL);
	signal (SIGTTOU, SIG_DFL);
	signal (SIGIO, SIG_DFL);
#ifndef	HPUX
	signal (SIGXCPU, SIG_DFL);
	signal (SIGXFSZ, SIG_DFL);
#endif
	signal (SIGVTALRM, SIG_DFL);
	signal (SIGPROF, SIG_DFL);
#else
#if	UNICOS
	signal (SIGUSR1, SIG_DFL);
	signal (SIGUSR2, SIG_DFL);
	signal (SIGCLD, SIG_DFL);
#else
BAD SYSTEM TYPE
#endif
#endif
	server = getenv ("SHELL");
#if	SGI | SYS52 | SYS53 | UTS | UNICOS
	if ((shellname = strrchr (server, '/')) != (char *) 0)
#else
#if	BSD42 | BSD43 | ULTRIX
	if ((shellname = rindex (server, '/')) != (char *) 0)
#else
BAD SYSTEM TYPE
#endif
#endif
		shellname += 1;         /* Step over last '/' */
	else shellname = server;        /* No '/' to step over */
	/*
	 *  Construct the proper argv array.
	 */
	sprintf (minusname, "-%s", shellname);
	argv [0] = minusname;
	argv [1] = (char *) 0;
	/*
	 *  Exec the proper shell.
	 */
	execve (server, argv, envp);
	/*
	 *  Fall through if execve() fails.
	 *
	 *  Write shell execve() diagnostic on the stderr
	 *  file of the batch request.
	 */
	fprintf (stderr, "FATAL:  Execve() of shell: %s failed.\n", server);
        fflush (stderr);
}
