/*++ mgr_movset.c - Network Queueing System
 *
 * $Source$
 *
 * DESCRIPTION:
 *
 *	NQS manager "move set" execution module.
 *
 *	Author:
 *	------
 *	David G. Evans, Cray Research Inc.
 *	August 11th, 1986.
 *
 *
 * STANDARDS VIOLATIONS:
 *   None.
 *
 * REVISION HISTORY: ($Revision$ $Date$ $State$)
 * $Log$
 *
 */

#include <stdio.h>		
#include "nqs.h"			/* NQS types and definitions */
#include "nqsmgr.h"			/* Token types and error messages */
#include "transactcc.h"			/* Transaction completion codes */

/*
 *	Declaration of transaction modules.
 */
extern long  movreq();			/* Move a request to a new queue */
extern long  modreq();			/* Modify a request */
/*
 *	Miscellaneous funtions
 */
extern char  *deq_set();		/* Dequeue text */
extern mid_t deq_mid();			/* Dequeue machine-id */
extern void  diagnose();		/* Diagnose completion code */
extern int   localmid();		/* Get local machine id */
extern int   scancpulim();		/* Parse a cpu time limit */
extern int   scanquolim();		/* Parse a quota limit */	
extern void  show_failed_prefix();	/* Show TCML_FAILURE prefix */


/*** modreq_set
 *
 *
 *	void modreq_set():
 *
 *	Modfify a request identifier. The set of modifiers is defined
 *	by the scan_reqmodset() function which is used to scan/parse
 *	the qualifier set specification.
 */
void modreq_set(req_seqno, orig_hostid, whomuid)
int req_seqno;				/* Request sequence number */
mid_t orig_hostid;			/* Originating machine identifier */
uid_t whomuid; 		                /* Calling user, else zero for */ 
                                        /*  NQS operator/manager call  */ 

{
	void modreq_problem();		/* Problem modifying a request */
	
	char *param_string;		/* Dequeued parameter string */
	long lim_type;			/* Request limit type - LIM_xxx */
	long param_value1;
	short param_value2;
	short param_infinite;
 
	short mssgflag = 0;		/* BOOLEAN TRUE if a message has /*
					/* been delivered to the user */
	register short exitflag = 0;	/* Exit flag */
	int qual_token;			/* Qualifier token */
	long comcode;			/* Completion code */

	while ((param_string = deq_set()) != NULL && !exitflag) {
		/*
		 *	Get qualifier type
		 */	
		qual_token = (int) deq_qual_type();
		/*
		 *	Analyze parameter value 
		 */
		switch (qual_token) {
		case 1:			/* Nice Value */
			param_value1 = atol (param_string);	
			if (param_value1 < (long) MIN_REQNICE || 
			    param_value1 > (long) MAX_REQNICE) {
				errormessage (EM_NICEOUTOFBOU);
				modreq_problem (req_seqno,orig_hostid);
				return;
			}
			param_value2 = 0;
			param_infinite = 0;
			lim_type = (long) LIM_PPNICE;
			break;
		case 2:			/* Priority Value */
			param_value1 = atol (param_string);	
			if (param_value1 < 0L || 
			    param_value1 > (long) MAX_RPRIORITY) {
				errormessage (EM_PRIOUTOFBOU);
				modreq_problem (req_seqno,orig_hostid);
				return;
			}
			param_value2 = 0;
			param_infinite = 0;
			lim_type = (long) LIM_PRIORITY;
			break;
		case 3:			/* CPU time limit */
			if (cput_valid (param_string, &param_value1,
			               &param_value2, &param_infinite) == -1) {
				modreq_problem (req_seqno,orig_hostid);
				return;
			}
			lim_type = (long) LIM_PRCPUT;
			break;
		case 4:			/* Memory Limit */	
			if (quota_valid (param_string, &param_value1,
			                &param_value2, &param_infinite) == -1) {
				modreq_problem (req_seqno,orig_hostid);
				return;
			}
			lim_type = (long) LIM_PRMEM;
			break;
		}	
		/*
		 *	Send packet to NQS daemon
		 */
		comcode = modreq ( whomuid, req_seqno, orig_hostid,
 			lim_type, param_value1, param_value2, param_infinite);
		switch (comcode) {
		case TCML_INTERNERR:
		case TCML_NOESTABLSH:
		case TCML_NOLOCALDAE:
		case TCML_PROTOFAIL:
			modreq_problem (req_seqno,orig_hostid);
			diagnose (comcode);
			mssgflag = 1;
			exitflag = 1;		/* Exit */
			break;
		case TCML_COMPLETE:
			break;
		case TCML_NOSUCHREQ:
		case TCML_REQRUNNING:
			modreq_problem (req_seqno,orig_hostid);
			diagnose (comcode);
			mssgflag = 1;
			exitflag = 1;		/* Exit */
			break;
		default:
			modreq_problem (req_seqno,orig_hostid);
			diagnose (TCML_UNDEFINED);
			mssgflag = 1;
			exitflag = 1;		/* Exit */
		}
	}
	if (!mssgflag) diagnose (TCML_COMPLETE);
}


/*** modreq_problem
 *
 *
 *	void modreq_problem()
 *	Indicate problem modifying a request.
 */
static void modreq_problem (seq_num, orig_hostid)
int seq_num;			/* Sequence number */
mid_t orig_hostid;			/* Request host machine id */
{
	show_failed_prefix();
	printf ("Problem modifying request: %ld.%s.\n",
		seq_num, getmacnam (orig_hostid));
}


/*** cput_valid
 *
 *
 *	int cput_valid ():
 *
 *	Validate a CPU time limit quota.
 *
 *	Returns:
 *		 0: if scanning was successful.
 *		-1: otherwise.
 */
static int cput_valid (cput_string, cpu_secs, cpu_msecs, infinite)
register char *cput_string;
long *cpu_secs;
short *cpu_msecs;
short *infinite;
{
	struct cpulimit cpulimit;

	*infinite = 0;					/* Finite value */
	if (*cput_string == 'u') {
		*cpu_secs = 0;			/* Give these fields */
		*cpu_msecs = 0;			/* values that will */
						/* pass integrity checks */
						/* checks */
		*infinite = 1;			/* Infinite */
	}
	else {
		switch (scancpulim (cput_string, &cpulimit, 1)) {
		case 0:
			*cpu_secs = cpulimit.max_seconds;
			*cpu_msecs = cpulimit.max_ms;
			break;
		case -1:
			errormessage (EM_INVCPULIMSYN);
			return (-1);
		case -2:
			errormessage (EM_BADCPULIMVAL);
			return (-1);
		case -3:
			errormessage (EM_CPULIMEXP);
			return (-1);
		}
	}
	return(0);			/* Scanning successful */
}


/*** quota_valid
 *
 *
 *	int quota_valid
 *
 *	Validate a quota limit having nothing to do
 *		 with "nice" or cpu limits.
 *
 *	Returns:
 *		 0: if scanning was successful, and privileges are ok;
 *		-1: otherwise.
 */
static int quota_valid (quota_string, quota_value1, quota_value2, infinite)
register char *quota_string;
long *quota_value1;
short *quota_value2;
short *infinite;
{
	struct quotalimit quotalimit;

	*infinite = 0;			/* Finite */
	if (*quota_string == 'u') {
		*quota_value1 = 0;		/* Give these fields */
		*quota_value2 = 0;		/* values that will */
						/* pass integrity */
						/* checks */
		*infinite = 1;			/* Infinite */
	}
	else {
		switch (scanquolim (quota_string, &quotalimit, 1)) {
		case 0:
			*quota_value1 = quotalimit.max_quota;
			*quota_value2 = quotalimit.max_units;
			break;
		case -1:
			errormessage (EM_INVQUOLIMSYN);
			return (-1);
		case -2:
			errormessage (EM_BADQUOLIMVAL);
			return (-1);
		case -3:
			errormessage (EM_QUOLIMEXP);
			return (-1);
		}
	}
	return (0);			/* Scanning successful */
}				

