/*++ nqs_failed.c - Network Queueing System
 *
 * $Source$
 *
 * DESCRIPTION:
 *
 *	Move the named request control file to the NQS hold directory
 *	for failed requests.
 *
 *
 *	Author:
 *	-------
 *	Brent A. Kingsbury, Sterling Software Incorporated.
 *	August 12, 1985.
 *
 *
 * STANDARDS VIOLATIONS:
 *   None.
 *
 * REVISION HISTORY: ($Revision$ $Date$ $State$)
 * $Log$
 *
 */

#include <stdio.h>
#include "nqs.h"			/* MAX_PATHNAME definition */
#include "nqsxvars.h"			/* Global vars and directories */
#if	SYS52 | SYS53 | UNICOS
#include <dirent.h>			/* Include directory walking defs */
#else
#ifdef	UTS
#include "dir.4.2.h"			/* Include directory walking defs */
#else
#if	BSD42 | BSD43 | SGI | SYS52 | SYS53 | ULTRIX
#include <sys/dir.h>			/* Include directory walking defs */
#else
BAD SYSTEM TYPE
#endif
#endif
#endif

/*
 *	External functions:
 */
extern int is6bitstr();			/* Is 6-bit string boolen fn */
extern void pack6name();		/* Compute name on 6-bit alphabet */
extern int strncmp();			/* String comparison */
extern int strlen();			/* Return string length */

/*** nqs_failed
 *
 *
 *	void nqs_failed():
 *
 *	Move the named request control file from the control directory
 *	to the holding directory for failed requests.
 */
void nqs_failed (orig_seqno, orig_mid)
long orig_seqno;			/* Sequence number for req */
mid_t orig_mid;				/* Machine-id for req */
{
	char ctrlname [15];		/* Name of control file in control */
					/* directory */
	char path [MAX_PATHNAME+1];	/* Control/data subdirectory path */
	char datapath [MAX_PATHNAME+1];	/* Data subdirectory path */
	char holdpath [MAX_PATHNAME+1];	/* Failed directory path */
	DIR *dir;			/* Open queue order dir structure */
#if	SYS52 | SYS53 | UNICOS
	struct dirent *dirent;		/* Ptr to directory entry */
#else
	struct direct *dirent;		/* Ptr to directory entry */
#endif

	printf ("I$Request being placed in failed directory.\n");
	fflush (stdout);
	pack6name (path, Nqs_control, (int) (orig_seqno % MAX_CTRLSUBDIRS),
		  (char *) 0, (long) orig_seqno, 5, (long) orig_mid, 6, 0, 0);
	pack6name (ctrlname, (char *) 0, -1, (char *) 0, (long) orig_seqno, 5,
		  (long) orig_mid, 6, 0, 0);
	sprintf (holdpath, "%s/%s", Nqs_failed, ctrlname);
	link (path, holdpath);		/* Establish link in failed dir */
	unlink (path);			/* Remove control subdirectory entry */
	/*
	 *  Traverse the appropriate data subdirectory to grab all data
	 *  files associated with the failed request.
	 */
	pack6name (datapath, Nqs_data, (int) (orig_seqno % MAX_DATASUBDIRS),
		  (char *) 0, 0L, 0, 0L, 0, 0, 0);
	if ((dir = opendir (datapath)) == (DIR *) 0) {
		printf ("E$Unable to open the data dir. in nqs_failed().\n");
		printf ("I$Data files for failed request ");
		printf ("not moved to failed directory.\n");
		fflush (stdout);
		return;
	}
#if	SYS52 | SYS53 | UNICOS
	while ((dirent = readdir (dir)) != (struct dirent *) 0) {
#else
	while ((dirent = readdir (dir)) != (struct direct *) 0) {
#endif
		/*
		 *  Determine if this data file is for the request being
		 *  placed on hold.
		 */
		if (strlen (dirent->d_name) == 14 &&
		    is6bitstr (dirent->d_name, 14) &&
		    strncmp (dirent->d_name, ctrlname, 11) == 0) {
			/*
			 *  This data file is for the request being placed
			 *  in the failed directory.
			 */
			sprintf (path, "%s/%s", datapath, dirent->d_name);
			sprintf (holdpath, "%s/%s", Nqs_failed,
				 dirent->d_name);
			link (path, holdpath);	/*Establish link to data file*/
			unlink (path);		/*Remove from data dir */
		}
	}
	closedir (dir);			/* Close the data file directory */
}
