/*++ nqs_family.c - Network Queueing System
 *
 * $Source$
 *
 * DESCRIPTION:
 *
 *	Process a "report process family" packet from a shepherd process.
 *
 *
 *	Author:
 *	-------
 *	Brent A. Kingsbury, Sterling Software Incorporated.
 *	August 12, 1985.
 *
 *
 * STANDARDS VIOLATIONS:
 *   None.
 *
 * REVISION HISTORY: ($Revision$ $Date$ $State$)
 * $Log$
 *
 *	August 7, 1991	Global batch run and pipe route limits
 *
 */

#include <stdio.h>
#include "nqs.h"			/* NQS constants and data types */
#include "nqsxvars.h"			/* NQS global variables */

extern void udb_reqpgrp();		/* Update request process-group */

/*** nqs_family
 *
 *
 *	void nqs_family():
 *	Process a "report process family" packet from a shepherd process.
 */
void nqs_family (shepherd_pid, process_family)
int shepherd_pid;			/* Process-id of shepherd process */
int process_family;			/* "Process family" of server */
{
	struct qhead *serverque;	/* Server was handling this queue */
	register struct qhead *queue;	/* Ptr to queue structure */
	register int runcount;		/* Reqs running in a queue */
	register struct request *req;	/* Request structure */
	register short reqindex;	/* Index in Runvars of the running */
					/* structure allocated for the */
					/* request under inspection */

	queue = Nonnet_queueset;		/* First queue in queue set */
	serverque = (struct qhead *)0;		/* Request queue not found */
	while (queue != (struct qhead *)0 && serverque == (struct qhead *)0) {
		runcount = queue->q.runcount;	/* # of requests running */
		req = queue->runset;		/* First run request */
		while (runcount--) {
			reqindex = req->reqindex;
			if ((Runvars + reqindex)->shepherd_pid == shepherd_pid) {
				/*
				 *  We have found the request!
				 */
				serverque = queue;
				runcount = 0;	/* Exit inner loop */
			}
			else req = req->next;
		}
		queue = queue->next;		/* Get next queue */
	}
	if (serverque == (struct qhead *) 0) {
		/*
		 *  The request was not located in the non-network
		 *  queue set.  Scan the network queue set for the
		 *  request.
		 */
		queue = Net_queueset;		/* First queue in queue set */
		serverque = (struct qhead *)0;	/* Request queue not found */
		while (queue != (struct qhead *) 0 &&
		       serverque == (struct qhead *) 0) {
			runcount = queue->q.runcount;
						/* # of requests running */
			req = queue->runset;	/* First run request */
			while (runcount--) {
				reqindex = req->reqindex;
				if ((Runvars + reqindex)->shepherd_pid
					== shepherd_pid) {
					/*
					 *  We have found the request!
					 */
					serverque = queue;
					runcount = 0;
						/* Exit inner loop */
				}
				else req = req->next;
			}
			queue = queue->next;	/* Get next queue */
		}
	}
	/*
	 *  Determine whether or not we found the request, and if it's
	 *  valid.
	 */
	if (serverque == (struct qhead *) 0 ||	/* Request not found */
	    (Runvars + reqindex)->process_family != 0 || /*Family known already!*/
	    process_family <= 0) {
		/*
		 *  Bad family reported.
		 */
		printf ("E$Bad process family reported.\n");
		fflush (stdout);
		return;
	}
	/*
	 *  Req points to the request structure for the request whose
	 *  "process family" is being reported.
	 *
	 *  Serverque points to the queue structure for the queue
	 *  containing the request.
	 */
	if (req->status & RQF_SIGQUEUED) {
		/*
		 *  We have a queued "kill -<signal> request".
		 *  Kill the server process group/family.
		 */
#if	SGI | SYS52 | SYS53 | UTS | UNICOS
		kill (-process_family, (Runvars + reqindex)->queued_signal);
#else
#if	BSD42 | BSD43 | ULTRIX
		killpg (process_family, (Runvars + reqindex)->queued_signal);
#else
BAD SYSTEM TYPE
#endif
#endif
		req->status &= ~RQF_SIGQUEUED;	/* Queued signal sent */
	}
	else {
		(Runvars + reqindex)->process_family = process_family;
						/* Store process-family */
		udb_reqpgrp (serverque, req);	/* Update the proper qentry */
	}					/* to show process group */
}
