/*++ nqs_fndnnq.c - Network Queueing System
 *
 * $Source$
 *
 * DESCRIPTION:
 *
 *	Return pointer to queue structure for the named non-network
 *	queue.
 *
 *
 *	Author:
 *	-------
 *	Brent A. Kingsbury, Sterling Software Incorporated.
 *	January 23, 1986.
 *
 *
 * STANDARDS VIOLATIONS:
 *   None.
 *
 * REVISION HISTORY: ($Revision$ $Date$ $State$)
 * $Log$
 *
 */

#include "nqs.h"
#include "nqsxvars.h"		/* External variables */

extern int strcmp();		/* String comparison */


/*** nqs_fndnnq
 *
 *
 *	struct qhead *nqs_fndnnq():
 *
 *	Return pointer to queue structure for the named non-network
 *	queue.
 *
 *	Returns:
 *		A pointer to the queue structure for the named non-
 *		network queue, if the named non-network queue exists;
 *		otherwise (struct qhead *)0 is returned.
 */
struct qhead *nqs_fndnnq (quename)
register char *quename;
{
	register struct qhead *queue;
	register int res;

	/*
	 *  We take advantage of the fact that the non-network queue set
	 *  is lexicographically ordered.
	 */
	queue = Nonnet_queueset;
	while (queue != (struct qhead *)0) {
		if ((res = strcmp (queue->q.namev.name, quename)) < 0) {
			/*
			 *  We have not yet found the queue, but we should
			 *  keep on searching.
			 */
			queue = queue->next;
		}
		else if (res == 0) return (queue);	/* Found! */
		else return ((struct qhead *)0);	/* Not found */
	}
	return (queue);					/* Not found */
}
