/*++ nqs_reqser.c - Network Queueing System
 *
 * $Source$
 *
 * DESCRIPTION:
 *
 *
 *	This module contains all of the logic associated with the
 *	request server created for each spawned NQS request.
 *
 *
 *	Author:
 *	-------
 *	Brent A. Kingsbury, Sterling Software Incorporated.
 *	May 5, 1986.
 *
 *
 * STANDARDS VIOLATIONS:
 *   None.
 *
 * REVISION HISTORY: ($Revision$ $Date$ $State$)
 * $Log$
 *
 *	April 9, 1991	Prologue/epilogue scripts
 *	April 25, 1991	Quota and cpu time limits for AT&T Unix systems
 *	July 8, 1991	Most of the signals ignored when building the job
 *	November 11, 1991 Several load-balancing queue
 *	April 13, 1992	Fill in a security hole concerning supplementary groups
 *
 */

#ifdef	DOMAIN_OS			/* Does not work with -D option */
#ifndef	QSUB_SHELL
#define QSUB_SHELL 1
#endif
#endif
#include <stdio.h>
#include <errno.h>
#include <pwd.h>			/* Used to determine GID */
#include <signal.h>			/* Signal definitions */
#include <sys/types.h>			/* Types definitions */
#include <sys/stat.h>			/* Stat definitions */
#include "nqs.h"			/* NQS constants and data types */
#include "nqspacket.h"			/* NQS local message packet types */
#include "nqsmail.h"			/* NQS mail definitions */
#include "nqsxvars.h"			/* NQS global variables and dirs */
#include "requestcc.h"			/* NQS request completion codes */
#if	SGI | SYS52 | SYS53 | UNICOS | UTS
#include <fcntl.h>
#else
#if	BSD42 | BSD43 | ULTRIX
#include <sys/file.h>
#else
BAD SYSTEM TYPE
#endif
#endif
#ifdef	_IBMESA
#define SEC_BASE 1
#include <sys/security.h>
#include <prot.h>
#endif

/*
 *
 *	External functions and variables.
 */
extern int errno;			/* System call error record */
extern char *asciierrno();		/* Return ASCII version of errno */
extern void enf_bsdcpu();		/* Enforce BSD4.2 time limit */
extern void enf_bsdnic();		/* Enforce BSD4.2 nice */
extern void enf_bsdquo();		/* Enforce BSD4.2 resource limit */
extern void enf_sy5cpu();		/* Enforce other OS time limit */
extern void enf_sy5nic();		/* Enforce other OS nice */
extern void enf_sy5quo();		/* Enforce other OS resource limit */
extern long errnototcm();		/* Errno to TCMx_ code */
extern char *fmtmidname();		/* Format machine name for mid */
extern char *fmttime();			/* Format date/time */
#if	SGI | SYS52 | SYS53 | UNICOS | UTS
extern char *getenv();			/* Get environment var */
#else
#if	BSD42 | BSD43 | ULTRIX
#else
BAD SYSTEM TYPE
#endif
#endif
extern int inter();			/* Send message packet */
extern void interclear();		/* Clear message packet */
extern void interw32i();		/* Write packet integer datum */
extern long lseek();			/* Lseek() */
extern int mai_send();			/* Send NQS request mail */
extern long mergertcm();		/* Merge transaction code (TCMx_) */
					/* with request completion code */
					/* (RCM_) */
extern char *namstderr();		/* Return name of temp stderr file */
extern char *namstdout();		/* Return name of temp stdout file */
extern char *nmap_get_nam();		/* Get principal name of machine */
extern void nqssleep();			/* NQS sleep() call */
extern int opendata();			/* Open an NQS data file */
extern int parseserv();			/* Parse server arguments */
extern void serexit();			/* Server exit procedure */
/*
 * Sprintf() returns an int on System V, a char * on BSD.
 */
extern char *strcpy();			/* String copy */
extern int strlen();			/* String length */
extern time_t time();
#if	SGI | SYS52 | SYS53 | UTS | UNICOS
extern char *strrchr();			/* Find last occurrence of char */
#else
#if	BSD42 | BSD43 | ULTRIX
extern char *rindex();			/* Find last occurrence of char */
#else
BAD SYSTEM TYPE
#endif
#endif
#ifdef	_IBMESA
extern int setluid();			/* Sets the login user ID (LUID) */
extern struct pr_passwd *getprpwuid();	/* Read protected password database */
extern int setpriv();			/* Sets the kernel authorization */
#endif


/*
 *	Variables local to this module.
 */
static long jid;			/* job ID */


/*** nqs_reqser
 *
 *
 *	void nqs_reqser():
 *
 *	We are the request server or shell process.
 *	We are the child of the shepherd process created
 *	in nqs_spawn.c.
 *
 *	  At this point:
 *		File descriptor 0: Control file (O_RDWR).
 *		File descriptor 1: Stdout writing to log process.
 *		File descriptor 2: Stderr writing to log process.
 *		File descriptor 3: Write file descriptor for sending
 *				   back request completion messages
 *				   from the server to the shepherd.
 *		File descriptor 4: Write file descriptor to NQS daemon
 *				   request pipe, this is enforced
 *				   by startup code in nqs_boot.c!
 *
 *	  If we are spawning a batch request with no specific shell,
 *	  and the shell strategy is FREE, then
 *
 *		File descriptor 5: Read file descriptor from the
 *				   shepherd process to the server
 *				   shell process containing the
 *				   user-process visible shell
 *				   script link name.
 *
 *	  otherwise:
 *
 *		File descriptor 5: is closed.
 *
 *	  In all cases:
 *
 *		File descriptors [6.._NFILE-1] are closed.
 *
 *		(On Berkeley UNIX implementations:
 *		 FILE descriptors [6..getdtablesize()-1] are closed.)
 *
 *
 *
 *	One of two distinct situations now exists:
 *
 *	  Case 1:  We are spawning a batch request with a shell
 *		   strategy of FREE.
 *
 *	.-----------------------.
 *	|  NQS daemon process	|<============================================.
 *	|			|<==.					     ||
 *	`-----------------------'  || Named-pipe	Named-pipe connection||
 *		    |		   || connected to	to the local NQS     ||
 *		    V		   || the NQS daemon	daemon request/event ||
 *	.-----------------------.  || request/event	pipe.		     ||
 *	|  Server shepherd	|>==' pipe.				     ||
 *	|  process.		|>=======================================.   ||
 *	|			|<==.					||   ||
 *	`-----------------------'  ||			Pipe from	||   ||
 *		    |		   || Serexit() pipe	shepherd process||   ||
 *		    V		   || TO shepherd	to shell with	||   ||
 *	.-----------------------.  || from shell.	scriptfile name.||   ||
 *	|  Shell process.	|>=='					||   ||
 *	|			|<======================================='   ||
 *	|			|>============================================'
 *	`-----------------------'
 *
 *
 *
 *	  Case 2:  We are spawning a batch request with a shell
 *		   strategy of FIXED or LOGIN (or the request
 *		   specifies a particular shell), or we are spawning
 *		   a device, network, or pipe request.
 * 
 *	.-----------------------.
 *	|  NQS daemon process	|<============================================.
 *	|			|<==.					     ||
 *	`-----------------------'  || Named-pipe	Named-pipe connection||
 *		    |		   || connected to	to the local NQS     ||
 *		    V		   || the NQS daemon	daemon request/event ||
 *	.-----------------------.  || request/event	pipe.		     ||
 *	|  Server shepherd	|>==' pipe.				     ||
 *	|  process.		|<==.					     ||
 *	`-----------------------'  ||					     ||
 *		    |		   || Serexit() pipe			     ||
 *		    V		   || TO shepherd			     ||
 *	.-----------------------.  || from shell.			     ||
 *	|  Shell process.	|>=='					     ||
 *	|			|>============================================'
 *	`-----------------------'
 *
 *
 */
void nqs_reqser (request, rawreq, restartflag, queue, device, passwd)
struct request *request;		/* Request structure */
struct rawreq *rawreq;			/* Raw request header from */
					/* control file */
int restartflag;			/* BOOLEAN request is restarting */
					/* execution */
struct qhead *queue;			/* Queue being served */
struct device *device;			/* Device to use (can be NIL) */
struct passwd *passwd;			/* Password entry for request */
{
	int mkspool();			/* Make a spooled output file */
	void serexecute();		/* Server executing request */

#if	MAX_QUEUENAME > MAX_DEVNAME
	char argv0buffer [MAX_QUEUENAME + 8];
#else
	char argv0buffer [MAX_DEVNAME + 8];
#endif
					/* Space for argv [0] name of */
					/* queue or device name + " server" */
	char environment [MAX_ENVIRONMENT];
					/* Space for environment */
	char restart_msg [80];		/* Batch request restart message */
	char *server;			/* Server and/or server arguments, */
					/* or pathname of batch shell */
	char *argv [MAX_SERVERARGS+1];	/* Execve() argument pointers */
	char *envp [MAX_ENVIRONVARS+1];	/* Execve() environment pointers */
	struct qdestmap *mapdest;	/* Used to walk destset for pipe-q */
	int fd;				/* File descriptor for device */
	int envpcount;			/* Number of environment vars */
	int envpsize;			/* Size of environment in chars */
	int filemask;			/* = O_CREAT if restartflag is TRUE */
					/* = O_CREAT | O_TRUNC if */
					/*   restartflag is FALSE */
	register int i;			/* Iteration counter & scratch var */
	register char *cp;		/* Character pointer */
	register int persist;		/* Persistent pipe queue request */
	/*
	 *  Variables specific to the spawning of a batch request.
	 */
	char minusname [257];		/* Name of shell (path removed) */
					/* prefixed with a "-" character */
	char buffer [MAX_REQPATH+2];	/* Batch request control file line */
					/* buffer */
	char *shellname;		/* Name of shell (path removed) */
	char *hostname;			/* Name of submitter's host */
	time_t timeval;			/* Current time */
	unsigned long retry_time;	/* Retry time */
	int script;			/* Shell script file descriptor */
	int standout;			/* Standard output file */
	int standerr;			/* Standard error file */
	static char *defshell =
#if	QSUB_SHELL
		"/bin/csh";
#else
		"/bin/sh";		/* By default : Bourne shell */
#endif
	static char *shellargs [2] =
#if	QSUB_SHELL
		{"csh", "-c"};
#else
		{"sh", "-c"};		/* Arguments for shell command */
#endif
	char shellcmd [257];		/* Shell command line */
	mid_t csmid [MAX_QDESTS];	/* Different destinations */
	mid_t tmid = 0;			/* First chosen destination */
	int openflags;
	int cmask;
	struct stat sbuf;
	char fullname [MAX_REQPATH+2];
	static char *retrymsg = "I$Device offline (%s:%s). retries = %d\n";
#ifdef	_IBMESA
	struct pr_passwd *acct;
#endif

	envpcount = 0;			/* No environment vars exist yet */
	envpsize = 0;
	if (queue->q.type != QUE_BATCH) {
		/*
		 *  Device, network, and pipe queue servers always have
		 *  a debug level environment variable.
		 */
		cp = environment + envpsize;
		envp [envpcount++] = cp;
		sprintf (cp, "DEBUG=%1d", Debug);
		envpsize += strlen (cp) + 1;
		if (queue->q.type != QUE_DEVICE) {
			/*
			 *  Network queue and pipe queue servers are
			 *  spawned with a real AND effective user-id
			 *  of root.  This is necessary so that the
			 *  ../lib/establish.c module can bind to a
			 *  system socket port (<1024).
			 *
			 *  These servers however must know the account
			 *  upon whose behalf they are operating.  This
			 *  information is passed to them through the
			 *  environment.
			 */
			cp = environment + envpsize;
			envp [envpcount++] = cp;
			sprintf (cp, "UID=%1d", passwd->pw_uid);
			envpsize += strlen (cp) + 1;
			cp = environment + envpsize;
			envp [envpcount++] = cp;
			sprintf (cp, "USERNAME=%s", passwd->pw_name);
			envpsize += strlen (cp) + 1;
		}
	}
#if	SGI | SYS52 | SYS53 | UNICOS | UTS
	/*
	 *  Network queue, pipe queue and batch queue servers for System
	 *  V based implementations of NQS always have a TZ (timezone)
	 *  environment variable (for ../lib/establish.c as invoked by
	 *  network and pipe queue servers).  (Please note that the TZ
	 *  environment variable is guaranteed to be present, since
	 *  nqs_boot.c checks for it.)
	 *
	 *  Berkeley style NQS implementations do not need or use TZ.
	 */
	cp = environment + envpsize;
	envp [envpcount++] = cp;
	envpsize += sprintf (cp, "TZ=%s", getenv ("TZ")) + 1;
#else
#if	BSD42 | BSD43 | ULTRIX
#else
BAD SYSTEM TYPE
#endif
#endif
	switch (queue->q.type) {
	case QUE_BATCH:
		/*
		 *  We are spawning a batch request.
		 */
		filemask = O_CREAT;
		if (!restartflag) {
			/*
			 *  This request is being spawned for the very
			 *  first time.  Set filemask to also include
			 *  O_TRUNC.
			 */
			filemask |= O_TRUNC;
		}
		/*
		 *  Open the shell script file.
		 */
		if (Shell_strategy == SHSTRAT_FREE &&
		    rawreq->v.bat.shell_name [0] == '\0') {
			/*
			 *  The batch request is being run with a shell
			 *  strategy of FREE.  File descriptor #5 has the
			 *  the user-process visible link name of the
			 *  shell script file for the request.
			 */
			script = 5;	/* Read on file-descr #5 */
		}
		else {
			/*
			 *  The batch request is being run with a shell
			 *  strategy of FIXED or LOGIN (or the request
			 *  specifies a particular shell).
			 */
			if ((script = opendata (rawreq->orig_seqno,
						rawreq->orig_mid, 1)) == -1) {
				/*
				 *  We were unable to open the shell script
				 *  file.
				 */
				if (errno == ENFILE) {
					serexit (RCM_ENFILERUN, (char *) 0);
				}
				serexit (RCM_BADCDTFIL,
					"Unable to open shell script file.");
			}
		}
		/*
		 *  Determine the principal name of the request owner's
		 *  machine as seen from the execution machine.
		 */
		errno = 0;		/* Clear errno so that we can tell */
					/* if a system call error occurred */
					/* in the nmap_get_nam() function */
		hostname = nmap_get_nam (rawreq->orig_mid);
		if (hostname == (char *) 0) {
			/*
			 *  Unable to determine the name of the
			 *  request owner's machine.
			 */
			if (errno == ENFILE) {
				serexit (RCM_ENFILERUN, (char *) 0);
			}
			/*
			 *  The request owner's machine-id is no
			 *  longer known to the execution machine.
			 */
			serexit (RCM_MIDUNKNOWN, (char *) 0);
		}
		/*
		 *  Build the login environment and select the proper
		 *  shell.
		 */
		cp = environment + envpsize;
		envp [envpcount++] = cp;
		sprintf (cp, "HOME=%s", passwd->pw_dir);
		envpsize += strlen (cp) + 1;
		cp = environment + envpsize;
		envp [envpcount++] = cp;
		if (rawreq->v.bat.shell_name [0] == '\0') {
		    /*
		     *  The batch request does not explicitly
		     *  specify a shell.  NQS must choose the
		     *  shell used to execute the batch request
		     *  based upon the configured shell
		     *  strategy of the local system.
		     */
		    if (Shell_strategy == SHSTRAT_FIXED) {
			/*
			 *  A shell strategy of FIXED is in
			 *  effect.
			 */
			sprintf (cp, "SHELL=%s", Fixed_shell);
		    }
		    else {
			/*
			 *  A shell strategy of FREE of LOGIN
			 *  is presently in effect.
			 */
			if (passwd->pw_shell == (char *) 0 ||
			    passwd->pw_shell [0] == '\0') {
			    /*
			     *  The default shell is the Bourne shell.
			     */
			    sprintf (cp, "SHELL=%s", "/bin/sh");
			}
			else {
			    /*
			     *  The password file entry for the owner
			     *  identifies a shell to be used.
			     */
			    sprintf (cp, "SHELL=%s", passwd->pw_shell);
			}
		    }
		}
		else {
		    /*
		     *  The request specifies that a specific shell
		     *  be used.
		     */
		    sprintf (cp, "SHELL=%s", rawreq->v.bat.shell_name);
		}
		envpsize += strlen (cp) + 1;
		/*
		 *  At this time, the character pointer variable: cp
		 *  refers to the SHELL= environment variable.
		 *
		 *  Set the variable:  server  to point to the
		 *  fully qualified pathname of the shell that
		 *  is going to be spawned.
		 */
		server = cp + 6;		/* Step over "SHELL=" */
		/*
		 *  Build the remaining login environment variable
		 *  set.
		 */
#if	SGI | SYS52 | SYS53 | UTS | UNICOS
		cp = environment + envpsize;
		envp [envpcount++] = cp;
		if (passwd->pw_uid == 0) {
			/*
			 *  We are running as root.  The default path
			 *  is slightly different.
			 */
			envpsize += sprintf (cp, "PATH=/bin:/etc:/usr/bin") +1;
		}
		else {
			/*
			 *  The request is not owned by root.
			 */
			envpsize += sprintf (cp, "PATH=:/bin:/usr/bin") + 1;
		}
		cp = environment + envpsize;
		envp [envpcount++] = cp;
		envpsize += sprintf (cp, "LOGNAME=%s", passwd->pw_name) + 1;
		cp = environment + envpsize;
		envp [envpcount++] = cp;
		envpsize += sprintf (cp, "MAIL=/usr/mail/%s",
				     passwd->pw_name) + 1;
		cp = environment + envpsize;
		envp [envpcount++] = cp;
		envpsize += sprintf (cp, "ENVIRONMENT=BATCH") + 1;
		if ((shellname = strrchr (server, '/')) != (char *) 0)
#else
#if	BSD42 | BSD43 | ULTRIX
		cp = environment + envpsize;
		envp [envpcount++] = cp;
		if (passwd->pw_uid == 0) {
			/*
			 *  We are running as root.  The default path
			 *  is slightly different.
			 */
			sprintf (cp, "PATH=/usr/ucb:/bin:/etc:/usr/bin");
		}
		else {
			/*
			 *  The request is not owned by root.
			 */
			sprintf (cp, "PATH=:/usr/ucb:/bin:/usr/bin");
		}
		envpsize += strlen (cp) + 1;
		cp = environment + envpsize;
		envp [envpcount++] = cp;
		sprintf (cp, "USER=%s", passwd->pw_name);
		envpsize += strlen (cp) + 1;
		cp = environment + envpsize;
		envp [envpcount++] = cp;
		sprintf (cp, "MAIL=/usr/spool/mail/%s", passwd->pw_name);
		envpsize += strlen (cp) + 1;
		cp = environment + envpsize;
		envp [envpcount++] = cp;
		sprintf (cp, "ENVIRONMENT=BATCH");
		envpsize += strlen (cp) + 1;
		if ((shellname = rindex (server, '/')) != (char *) 0)
#else
BAD SYSTEM TYPE
#endif
#endif
									{
			shellname += 1;		/* Step over last '/' */
		}
		else shellname = server;	/* No '/' to step over */
		/*
		 *  Construct the proper argv array.
		 */
		sprintf (minusname, "-%s", shellname);
		argv [0] = minusname;
		argv [1] = (char *) 0;
		/*
		 *  Read the varying portion of the control file as a
		 *  sequence of lines terminated with newline characters,
		 *  adding environment variables as appropriate.
		 */
		fseek (stdin, lseek (fileno (stdin), 0L, 1), 0);
		while (!feof (stdin) &&
			fgets (buffer, MAX_REQPATH+2, stdin) != (char *) 0) {
		    i = strlen (buffer) - 1;	/* Get length of buffer -1 */
		    if (buffer [i] != '\n') {
			/*
			 *  We did not see a new line character.
			 *  The control file line is too long.
			 */
			serexit (RCM_BADCDTFIL,
				 "Varying control file line too long.");
		    }
		    buffer [i] = '\0';		/* Delete the newline char */
		    if (buffer [0] == 'D') {
			/*
			 *  This line has the value for the environment
			 *  variable:  QSUB_WORKDIR.
			 */
			i--;		/* i now equals the length of the */
					/* QSUB_WORKDIR directory string*/
			/*
			 *  Check to see if the environment variables:
			 *  QSUB_HOST, QSUB_REQID, QSUB_REQNAME,
			 *  and QSUB_WORKDIR will fit.
			 */
			if (envpcount + 4 >= MAX_ENVIRONVARS ||
			    strlen (hostname) * 2 + envpsize + 59 + i +
			    strlen (rawreq->reqname) > MAX_ENVIRONMENT) {
				/*
				 *  There is not sufficient space in
				 *  the environment array for all of
				 *  this information.
				 */
				serexit (RCM_2MANYENVARS, (char *) 0);
			}
			/*
			 *  Create the "QSUB_HOST=" environment variable.
			 *  QSUB_HOST=hostname
			 */
			cp = environment + envpsize;
			envp [envpcount++] = cp;
			sprintf (cp, "QSUB_HOST=%s", hostname);
			envpsize += strlen (cp) + 1;
			/*
			 *  Create the "QSUB_REQID=" environment variable.
			 *  QSUB_REQID=nnnnn.hostname
			 */
			cp = environment + envpsize;
			envp [envpcount++] = cp;
			sprintf (cp, "QSUB_REQID=%1ld.%s",
					    (long) rawreq->orig_seqno,
					     hostname);
			envpsize += strlen (cp) + 1;
			/*
			 *  Create the "QSUB_REQNAME=" environment variable.
			 *  QSUB_REQNAME=reqname
			 */
			cp = environment + envpsize;
			envp [envpcount++] = cp;
			sprintf (cp, "QSUB_REQNAME=%s", rawreq->reqname);
			envpsize += strlen (cp) + 1;
			/*
			 *  Create the "QSUB_WORKDIR=" environment variable.
			 *  QSUB_WORKDIR=workdir
			 */
			cp = environment + envpsize;
			envp [envpcount++] = cp;
			sprintf (cp, "QSUB_WORKDIR=%s", buffer+1);
			envpsize += strlen (cp) + 1;
		    }
		    else if (buffer [0] == 'E') {
			/*
			 *  This is a simple environment variable.
			 *  Get space for the environment variable
			 *  (i has the length of the entire environment
			 *  string + 1), and store the environment
			 *  string.
			 */
			if (envpcount >= MAX_ENVIRONVARS ||
			    envpsize + i > MAX_ENVIRONMENT) {
				/*
				 *  There is not sufficient space in
				 *  the environment array.
				 */
				serexit (RCM_2MANYENVARS, (char *) 0);
			}
			cp = environment + envpsize;
			envp [envpcount++] = cp;
			strcpy (cp, buffer+1);
			envpsize += i;
		    }
		    /*
		     *  File staging event specifications: [I,O]
		     *  are, and should always be, ignored here.
		     *  Their implementation will always be done
		     *  elsewhere.
		     */
		}
		/*
		 *  If the output file is to be spooled, then create the
		 *  temporary version of it in the NQS output spooling
		 *  directory BEFORE we chdir() to the user's home
		 *  directory.
		 *
		 *  It is possible that NQS will someday be linked
		 *  with a Newcastle or equivalent distributed file
		 *  system library, which intercepts system calls always
		 *  passing ABSOLUTE file paths to the kernel on the
		 *  appropriate machine.  Since the spool files reside
		 *  within the NQS protected file hierarchy, it is
		 *  necessary to be running as root, when a spool file
		 *  is being created under Newcastle-like file system
		 *  implementations.
		 */
		if (rawreq->v.bat.stdout_acc & OMD_SPOOL) {
			/*
			 *  The output file is to be spooled.
			 */
			standout = mkspool (rawreq, namstdout, passwd,
					    filemask);
			if (standout == -1) {
				/*
				 *  Alas, we failed!
				 */
				if (errno == ENFILE) {
					/*
					 *  The system file table is full.
					 */
					serexit (RCM_ENFILERUN, (char *) 0);
				}
				if (errno == ENOSPC) {
					/*
					 *  Insufficient space.
					 */
					serexit (RCM_ENOSPCRUN, (char *) 0);
				}
				serexit (RCM_UNAFAILURE,
					"Unable to create spooled stdout.");
			}
		}
		/*
		 *  If the error file is to be spooled, then create the
		 *  temporary version of it in the NQS output spooling
		 *  directory BEFORE we chdir() to the user's home
		 *  directory.
		 */
		if ((rawreq->v.bat.stderr_acc & OMD_SPOOL) &&
		    (rawreq->v.bat.stderr_acc & OMD_EO) == 0) {
			/*
			 *  The error file is to be spooled.
			 */
			standerr = mkspool (rawreq, namstderr, passwd,
					    filemask);
			if (standerr == -1) {
				/*
				 *  Alas, we failed!
				 */
				if (errno == ENFILE) {
					/*
					 *  The system file table is full.
					 */
					serexit (RCM_ENFILERUN, (char *) 0);
				}
				if (errno == ENOSPC) {
					/*
					 *  Insufficient space.
					 */
					serexit (RCM_ENOSPCRUN, (char *) 0);
				}
				serexit (RCM_UNAFAILURE,
					"Unable to create spooled stderr.");
			}
		}
		break;
	case QUE_DEVICE:
		/*
		 *  The request is a device request.  Try to open the device
		 *  for reading AND writing.  If this fails, then try to
		 *  open the device for writing only.  If that fails, then
		 *  go through a wait/retry process until the retry limit
		 *  is reached.
		 */
		close (1);		/* Stdout is going to become the */
					/* device leaving only stderr for */
					/* catastrophic error reporting */
		i = 0;
		openflags = 0;
		if ((stat (device->fullname, &sbuf)) == -1) {
			serexit(RCM_DEVOPEFAI, asciierrno());
		}
		if ((sbuf.st_mode & S_IFMT) == S_IFDIR) {
			sprintf (fullname,"%s/%ld.%s", device->fullname,
				(long) rawreq->orig_seqno,
				getmacnam (rawreq->orig_mid));
			cmask = umask(0177);
			openflags = O_CREAT;
		} else
			strcpy (fullname, device->fullname);
		do {
			if ((fd = open (fullname, O_RDWR | openflags,
					0777)) == -1) {
				/*
				 *  Would not open for read and write.
				 *  Try just write.
				 */
				if ((fd = open (fullname, O_WRONLY | openflags,
						0777)) == -1) {
					if (i < Maxoperet) {
						fprintf (stderr, retrymsg,
							device->name,
							fullname, i+1);
						fflush (stderr);
						if (Opewai == 0) nqssleep (1);
						else nqssleep (Opewai);
					}
					i++;	/* Retry count */
				}
			}
		} while (i <= Maxoperet && fd == -1);
		if (fd == -1) serexit (RCM_DEVOPEFAI, asciierrno ());
		if ((sbuf.st_mode & S_IFMT) == S_IFDIR) {
			/*
			 *  Make sure the user pays for the spooled
			 *  disk space (it was created while running
			 *  under root).
			 */
			umask (cmask);
			chown (fullname, passwd->pw_uid, passwd->pw_gid);
		}
		/*
		 *  Make sure that the device is open as stdout (fd#1).
		 */
		if (fd != 1) {
			fcntl (fd, F_DUPFD, 1);
			close (fd);
		}
		/*
		 *  We have the device open.
		 *  Get the name of the server and args to execve.
		 */
		server = device->server;
		break;
	case QUE_NET:
		/*
		 *  We are spawning a network queue request.
		 */
		cp = environment + envpsize;
		envp [envpcount++] = cp;
		sprintf (cp, "DEFAULT_RETRYTIME=%1ld", (long) Defnetrettim);
		envpsize += strlen (cp) + 1;
		cp = environment + envpsize;
		envp [envpcount++] = cp;
		sprintf (cp, "DEFAULT_RETRYWAIT=%1ld", (long) Defnetretwai);
		envpsize += strlen (cp) + 1;
		cp = environment + envpsize;
		envp [envpcount++] = cp;
		sprintf (cp, "ELAPSED_RETRYTIME=0");
		envpsize += strlen (cp) + 1;
		cp = environment + envpsize;
		envp [envpcount++] = cp;
		sprintf (cp, "OP=O");		/* Output file transaction */
		envpsize += strlen (cp) + 1;
		cp = environment + envpsize;
		envp [envpcount++] = cp;
		sprintf (cp, "EVENT=%1d", request->v1.subreq.event);
		server = queue->q.v1.network.server;
		break;				/* Get server/args to execve */
	case QUE_PIPE:
		/*
		 *  We are spawning a pipe queue request.
		 *  Get the server, and format the destination set
		 *  into the environment for the pipe queue server.
		 */
		cp = environment + envpsize;
		envp [envpcount++] = cp;
		sprintf (cp, "DEFAULT_RETRYTIME=%1ld", (long) Defdesrettim);
		envpsize += strlen (cp) + 1;
		cp = environment + envpsize;
		envp [envpcount++] = cp;
		sprintf (cp, "DEFAULT_RETRYWAIT=%1ld", (long) Defdesretwai);
		envpsize += strlen (cp) + 1;
		i = 0;				/* Destination# */
		mapdest = queue->v1.pipe.destset;
		while (mapdest != (struct qdestmap *) 0) {
			csmid [i++] = mapdest->pipeto->rhost_mid;
		    	mapdest = mapdest->next;/* Ordered destinations */
		}
		if (strncmp (CPIPQUENAM, queue->q.namev.name,
			     strlen (CPIPQUENAM)) == 0)
			/*
			 *  For load-balancing queues, we use the request
			 *  number as an index in the queue destination list.
			 */
			tmid = csmid [rawreq->orig_seqno % i];
		else
			tmid = csmid [0];
						/* Choose first destination */
		i = 0;				/* Destination# */
		persist = 0;			/* Unassuming request */
		timeval = time ((time_t *) 0);	/* Current time */
		mapdest = queue->v1.pipe.destset;
		while (mapdest != (struct qdestmap *) 0) {
		  if (mapdest->pipeto->rhost_mid >= tmid) {
		    if (mapdest->pipeto->status & DEST_ENABLED) {
			/*
			 *  The destination is enabled.
			 *  Add this destination to the
			 *  set of pipe queue destinations
			 *  in the environment.
			 *
			 *  Note that no explicit checks are
			 *  made here to see if we are running
			 *  off the end of the environment arrays!
			 *
			 *  We are depending on the #defines in
			 *  nqs.h to have been honestly defined.
			 */
			retry_time = 0;
			if ((mapdest->pipeto->status & DEST_RETRY) &&
			     timeval > mapdest->pipeto->retrytime) {
			    retry_time = (unsigned long) timeval
				- (unsigned long) mapdest->pipeto->retrytime;
			}
			cp = environment + envpsize;
			envp [envpcount++] = cp;
			sprintf (cp, "D%03d=%1ld %1lu %s", i++,
				(long) mapdest->pipeto->rhost_mid,
				 retry_time, mapdest->pipeto->rqueue);
			envpsize += strlen (cp) + 1;
		    }
		    else {
			/*
			 *  A destination exists that is presently disabled
			 *  for this pipe queue.  Since not all destinations
			 *  are therefore available to the pipe queue server
			 *  (pipeclient), the request should be allowed to
			 *  persist even if none of the enabled destinations
			 *  accept it, since it is not known that all possible
			 *  destinations for request have been tried.
			 */
			persist = 1;		/* Let the request persist */
		    }
		  }
		  mapdest = mapdest->next;	/* Get the next destination */
		}
		mapdest = queue->v1.pipe.destset;
		while (mapdest != (struct qdestmap *) 0) {
		  if (mapdest->pipeto->rhost_mid < tmid) {
		    if (mapdest->pipeto->status & DEST_ENABLED) {
			/*
			 *  The destination is enabled.
			 *  Add this destination to the
			 *  set of pipe queue destinations
			 *  in the environment.
			 *
			 *  Note that no explicit checks are
			 *  made here to see if we are running
			 *  off the end of the environment arrays!
			 *
			 *  We are depending on the #defines in
			 *  nqs.h to have been honestly defined.
			 */
			retry_time = 0;
			if ((mapdest->pipeto->status & DEST_RETRY) &&
			     timeval > mapdest->pipeto->retrytime) {
			    retry_time = (unsigned long) timeval
				- (unsigned long) mapdest->pipeto->retrytime;
			}
			cp = environment + envpsize;
			envp [envpcount++] = cp;
			sprintf (cp, "D%03d=%1ld %1lu %s", i++,
				(long) mapdest->pipeto->rhost_mid,
				 retry_time, mapdest->pipeto->rqueue);
			envpsize += strlen (cp) + 1;
		    }
		    else {
			/*
			 *  A destination exists that is presently disabled
			 *  for this pipe queue.  Since not all destinations
			 *  are therefore available to the pipe queue server
			 *  (pipeclient), the request should be allowed to
			 *  persist even if none of the enabled destinations
			 *  accept it, since it is not known that all possible
			 *  destinations for request have been tried.
			 */
			persist = 1;		/* Let the request persist */
		    }
		  }
		  mapdest = mapdest->next;	/* Get the next destination */
		}
		if (persist) {
		    /*
		     *  The request should be allowed to persist, even
		     *  if no destination accepts it this time.
		     */
		    cp = environment + envpsize;
		    envp [envpcount++] = cp;
		    strcpy (cp, "PERSIST=Y");
		}
		server = queue->q.v1.pipe.server;
	}					/* Get server/args to execve */
	/*
	 *  Place the NIL pointer at the end of the environment var
	 *  list for the server.
	 */
	envp [envpcount] = (char *) 0;
	/*
	 *  In ALL cases, the current working directory is the NQS
	 *  root directory, and the appropriate environment has been
	 *  constructed for the server.
	 */
	if (queue->q.type != QUE_BATCH) {
		/*
		 *  A device, network, or pipe queue request is
		 *  being spawned.
		 */
		if (parseserv (server, argv) == -1) {	/* Break into args */
			/*
			 *  Too many server arguments in server command line.
			 */
			serexit (RCM_2MANYSVARGS, (char *) 0);
		}
		server = argv [0];	/* Get ptr to program to execve */
		if (queue->q.type == QUE_DEVICE) cp = device->name;
		else if (queue->q.type == QUE_NET) {
			cp = fmtmidname (queue->q.namev.to_destination);
		}
		else cp = queue->q.namev.name;
		argv [0] = argv0buffer;
		sprintf (argv [0], "%s %s", cp, "server");
	}
	/*
	 *  Show debugging information if required.
	 */
	if (Debug) {
		/*
		 *  Display debug information.  Note that we have to use
		 *  stderr, since stdout is used for devices....
		 */
		fprintf (stderr, "D$nqs_reqser().\n");
		fprintf (stderr, "D$Execve'ing server: %s.\n", server);
		fflush (stderr);
		if (Debug > 1) {
			i = 0;
			while (argv [i] != (char *) 0) {
				fprintf (stderr, "D$argv[%1d]=%s.\n", i,
					 argv [i]);
				fflush (stderr);
				i++;
			}
			i = 0;
			while (envp [i] != (char *) 0) {
				fprintf (stderr, "D$envp[%1d]=%s.\n", i,
					 envp [i]);
				fflush (stderr);
				i++;
			}
		}
	}
	/*
	 *  Send mail as necessary notifying the user that their request
	 *  is beginning or restarting execution.
	 */
	if (queue->q.type == QUE_BATCH || queue->q.type == QUE_DEVICE) {
		if (restartflag) {	/* Restarting execution */
			if (rawreq->flags & RQF_RESTARTMAIL) {
				mai_send (rawreq, (struct requestlog *) 0,
					  MSX_RESTARTING);
			}
		}
		else {			/* Beginning execution for the first */
			if (rawreq->flags & RQF_BEGINMAIL) {	/* time */
				mai_send (rawreq, (struct requestlog *) 0,
					  MSX_BEGINNING);
			}
		}
	}
	/*
	 *  Set nice value for batch requests.
	 */
	if (queue->q.type == QUE_BATCH) {
#if	(VALID_LIMITS & LIM_PPNICE)
#if	BSD42 | BSD43 | ULTRIX
		enf_bsdnic (rawreq->v.bat.ppnice);
#else
#if	SGI | SYS52 | SYS53 | UTS | UNICOS
		enf_sy5nic (rawreq->v.bat.ppnice);
#else
BAD SYSTEM TYPE
#endif
#endif
#endif
	}
	/*
	 *  Set ourselves in our very own process group/family so that
	 *  the NQS daemon can kill us (and our children) as a group if
	 *  necessary.
	 */
	jid = getpid();
#if	BSD42 | BSD43 | _IBMESA | OSF1 | ULTRIX
	setpgrp (0, jid);		/* Process group = pid */
#else
#if	SGI | SYS52 | SYS53 | UTS | UNICOS
	setpgrp ();			/* Process group = pid */
#else
BAD SYSTEM TYPE
#endif
#endif
	if (queue->q.type == QUE_BATCH) {
		/*
		 *  We are spawning a batch request.
		 *  Enforce resource limits on the process or process group
		 *  while we're still root.
		 */
#if		(VALID_LIMITS & LIM_PPCORE)
#if		BSD42 | BSD43 | _IBMESA | OSF1 | SOLARIS | ULTRIX | SGI
		enf_bsdquo (&rawreq->v.bat.ppcoresize,
			    rawreq->v.bat.infinite & LIM_PPCORE, LIM_PPCORE);
#else
#if		SYS53
		enf_sy5quo (&rawreq->v.bat.ppcoresize,
			    rawreq->v.bat.infinite & LIM_PPCORE, LIM_PPCORE);
#else
#if		SYS52 | UTS | UNICOS
#else
BAD SYSTEM TYPE
#endif
#endif
#endif
#endif

#if		(VALID_LIMITS & LIM_PPDATA)
#if		BSD42 | BSD43 | _IBMESA | OSF1 | SOLARIS | ULTRIX | SGI
		enf_bsdquo (&rawreq->v.bat.ppdatasize,
			    rawreq->v.bat.infinite & LIM_PPDATA, LIM_PPDATA);
#else
#if		SYS53
		enf_sy5quo (&rawreq->v.bat.ppdatasize,
			    rawreq->v.bat.infinite & LIM_PPDATA, LIM_PPDATA);
#else
#if		SYS52 | UTS | UNICOS
#else
BAD SYSTEM TYPE
#endif
#endif
#endif
#endif

#if		(VALID_LIMITS & LIM_PPPFILE)
#if		BSD42 | BSD43 | _IBMESA | OSF1 | SOLARIS | ULTRIX | SGI
		enf_bsdquo (&rawreq->v.bat.pppfilesize,
			    rawreq->v.bat.infinite & LIM_PPPFILE, LIM_PPPFILE);
#else
#if		SYS52 | SYS53 | UTS | UNICOS
		enf_sy5quo (&rawreq->v.bat.pppfilesize,
			    rawreq->v.bat.infinite & LIM_PPPFILE, LIM_PPPFILE);
#else
BAD SYSTEM TYPE
#endif
#endif
#endif

#if		(VALID_LIMITS & LIM_PPMEM)
#if		UNICOS
		enf_sy5quo (&rawreq->v.bat.ppmemsize,
			    rawreq->v.bat.infinite & LIM_PPMEM, LIM_PPMEM);
#else
#if		BSD42 | BSD43 | ULTRIX | SGI | SYS52 | SYS53 | UTS
#else
BAD SYSTEM TYPE
#endif
#endif
#endif

#if		(VALID_LIMITS & LIM_PRMEM)
#if		UNICOS
		enf_sy5quo (&rawreq->v.bat.prmemsize,
			    rawreq->v.bat.infinite & LIM_PRMEM, LIM_PRMEM);
#else
#if		BSD42 | BSD43 | ULTRIX | SGI | SYS52 | SYS53 | UTS
#else
BAD SYSTEM TYPE
#endif
#endif
#endif

#if		(VALID_LIMITS & LIM_PPSTACK)
#if		BSD42 | BSD43 | _IBMESA | OSF1 | SOLARIS | ULTRIX | SGI
		enf_bsdquo (&rawreq->v.bat.ppstacksize,
			    rawreq->v.bat.infinite & LIM_PPSTACK, LIM_PPSTACK);
#else
#if		SYS53
		enf_sy5quo (&rawreq->v.bat.ppstacksize,
			    rawreq->v.bat.infinite & LIM_PPSTACK, LIM_PPSTACK);
#else
#if		SYS52 | UTS | UNICOS
#else
BAD SYSTEM TYPE
#endif
#endif
#endif
#endif

#if		(VALID_LIMITS & LIM_PPWORK)
#if		BSD42 | BSD43 | _IBMESA | OSF1 | SOLARIS | ULTRIX | SGI
		enf_bsdquo (&rawreq->v.bat.ppworkset,
			    rawreq->v.bat.infinite & LIM_PPWORK, LIM_PPWORK);
#else
#if		SYS53
		enf_sy5quo (&rawreq->v.bat.ppworkset,
			    rawreq->v.bat.infinite & LIM_PPWORK, LIM_PPWORK);
#else
#if		SYS52 | UTS | UNICOS
#else
BAD SYSTEM TYPE
#endif
#endif
#endif
#endif

#if		(VALID_LIMITS & LIM_PPCPUT)
#if		BSD42 | BSD43 | ULTRIX | SGI | SYS53
		enf_bsdcpu (&rawreq->v.bat.ppcputime,
			    rawreq->v.bat.infinite & LIM_PPCPUT, LIM_PPCPUT);
#else
#if		UNICOS
		enf_sy5cpu (&rawreq->v.bat.ppcputime,
			    rawreq->v.bat.infinite & LIM_PPCPUT, LIM_PPCPUT);
#else
#if		SYS52 | UTS
#else
BAD SYSTEM TYPE
#endif
#endif
#endif
#endif

#if		(VALID_LIMITS & LIM_PRCPUT)
#if		UNICOS
		enf_sy5cpu (&rawreq->v.bat.prcputime,
			    rawreq->v.bat.infinite & LIM_PRCPUT, LIM_PRCPUT);
#else
#if		BSD42 | BSD43 | ULTRIX | SGI | SYS52 | SYS53 | UTS
#else
BAD SYSTEM TYPE
#endif
#endif
#endif

#if		(VALID_LIMITS & LIM_PPNICE)
#if		BSD42 | BSD43 | ULTRIX
		enf_bsdnic (rawreq->v.bat.ppnice);
#else
#if		SGI | SYS52 | SYS53 | UTS | UNICOS
		enf_sy5nic (rawreq->v.bat.ppnice);
#else
BAD SYSTEM TYPE
#endif
#endif
#endif
	}
	if (queue->q.type == QUE_BATCH || queue->q.type == QUE_DEVICE) {
		/*
		 *  Batch and device requests must now set their real and
		 *  effective user and group-ids to that of the request
		 *  owner.
		 *
		 *  Network queue and pipe queue servers must NOT do this.
		 *  Network queue and pipe queue servers must be spawned
		 *  as root.
		 */
#ifdef	_IBMESA
		setluid (passwd->pw_uid);	/* Set login id */
#endif
		setgid ((int) passwd->pw_gid);	/* Set login group-id */
#if	BSD42 | BSD43 | ULTRIX | SYS52 | SYS53
		initgroups (passwd->pw_name, passwd->pw_gid);
#endif
		setuid ((int) passwd->pw_uid);	/* Set mapped user-id */
#ifdef	_IBMESA
		/* Establishing Kernel auths */
		acct = getprpwuid (passwd->pw_uid);
		if (acct->uflg.fg_sprivs)
			setpriv (SEC_KERNEL_AUTHS, acct->ufld.fd_sprivs);
		else
			if (acct->sflg.fg_sprivs)
				setpriv (SEC_KERNEL_AUTHS,
					 acct->sfld.fd_sprivs);
			else
				setpriv (SEC_KERNEL_AUTHS, privvec(-1));
#endif
	}
	/*
	 *  Explicitly set the signal receipt actions to be ignored so
	 *  that the server gets a completely "clean" signal environment.
	 *  Only in the child process which really handles the job, they
	 *  are set to their defaults (see ./execsh.c).
	 *  (Note that in most of the cases, except for SIGKILL and SIGSTOP,
	 *  the epilogue script is executed.)
	 */
	signal (SIGHUP, SIG_IGN);
	signal (SIGINT, SIG_IGN);
	signal (SIGQUIT, SIG_IGN);
	signal (SIGILL, SIG_IGN);
	signal (SIGTRAP, SIG_IGN);
#if	BSD42 | BSD43 | ULTRIX | SGI | SYS52 | SYS53 | UTS
	signal (SIGIOT, SIG_IGN);
	signal (SIGEMT, SIG_IGN);
#else
#if	UNICOS
	signal (SIGHWE, SIG_IGN);
#else
BAD SYSTEM TYPE
#endif
#endif
	signal (SIGFPE, SIG_IGN);
#if	BSD42 | BSD43 | ULTRIX | SGI | SYS52 | SYS53 | UTS
	signal (SIGBUS, SIG_DFL);
	signal (SIGSEGV, SIG_DFL);
#else
#if	UNICOS
	signal (SIGPRE, SIG_DFL);
	signal (SIGORE, SIG_DFL);
#else
BAD SYSTEM TYPE
#endif
#endif
	signal (SIGSYS, SIG_IGN);
	signal (SIGPIPE, SIG_IGN);
	signal (SIGALRM, SIG_IGN);
	signal (SIGTERM, SIG_IGN);
#if	SYS52 | UTS
	signal (SIGUSR1, SIG_IGN);
	signal (SIGUSR2, SIG_IGN);
	signal (SIGCLD, SIG_IGN);
	signal (SIGPWR, SIG_IGN);
#else
#if	BSD42 | BSD43 | ULTRIX | SGI | SYS53
	signal (SIGURG, SIG_IGN);
	signal (SIGTSTP, SIG_IGN);
	signal (SIGCONT, SIG_IGN);
	signal (SIGCHLD, SIG_IGN);
	signal (SIGTTIN, SIG_IGN);
	signal (SIGTTOU, SIG_IGN);
	signal (SIGIO, SIG_IGN);
#ifndef	HPUX
	signal (SIGXCPU, SIG_IGN);
	signal (SIGXFSZ, SIG_IGN);
#endif
	signal (SIGVTALRM, SIG_IGN);
	signal (SIGPROF, SIG_IGN);
#else
#if	UNICOS
	signal (SIGUSR1, SIG_IGN);
	signal (SIGUSR2, SIG_IGN);
	signal (SIGCLD, SIG_IGN);
#else
BAD SYSTEM TYPE
#endif
#endif
#endif
	/*
	 *  ***TRIAGE***
	 *
	 *  Extreme schedule constraints have forced us to leave
	 *  network queues unimplemented (software implementation
	 *  by triage).  When the day arrives that network queues
	 *  are a reality, then the obvious code section below
	 *  should be changed to read:
	 *  
	 *	/*
	 *	 *  Report our "process-family" to the NQS daemon.
	 *	 */
	/*	interclear();
	 *	interw32i ((long) getppid());	/* Shepherd process-id */
	/*	interw32i ((long) getpid());	/* Our process-id */
	/*	inter (PKT_FAMILY);
	 *
	 *  WITHOUT the surrounding "if" statement that PREVENTS
	 *  network queue requests from sending PKT_FAMILY packets
	 *  to the local NQS daemon.
	 */
	if (queue->q.type != QUE_NET) {		/* THIS IF STATEMENT SHOULD */
						/* EVENTUALLY BE REMOVED */
						/* (SEE ABOVE COMMENTS) */
		/*
		 *  Report our "process-family" to the NQS daemon.
		 */
		interclear();
		interw32i ((long) getppid());	/* Shepherd process-id */
		interw32i ((long) jid);		/* Our job-id */
		inter (PKT_FAMILY);
	}					/* (SEE ABOVE COMMENTS) */
	/*
	 *  Batch and device requests are not allowed to retain their
	 *  connection to the NQS daemon event/request pipe.  Only pipe
	 *  and network queue servers need to be able to talk directly to
	 *  the NQS daemon.
	 */
	if (queue->q.type == QUE_BATCH || queue->q.type == QUE_DEVICE) {
		close (Write_fifo);
	}
	/*
	 *  Batch requests are handled differently....
	 */
	if (queue->q.type == QUE_BATCH) {
		/*
		 *  Change directory to the user's home directory.
		 */
		if (chdir (passwd->pw_dir) == -1) {
			/*
			 *  We were unable to chdir() to the user's
			 *  home directory.
			 */
			serexit (RCM_UNABLETOEXE,
				 "Unable to chdir() to user home directory.");
		}
		/*
		 *  Set the file creation mask to the value that it
		 *  had when the user first submitted the request, with
		 *  exception that the umask will NOT be allowed to
		 *  disable WRITE access for the request owner.
		 *
		 *  We do this so that any stdout or stderr output file
		 *  created at this point, will be created such that it
		 *  will be writeable by the request owner.  This is
		 *  required if the request is to be restartable, and
		 *  a stdout or stderr file exists from a previously
		 *  interrupted execution of the batch request.
		 *
		 *  The umask will be set precisely later on.
		 */
		umask (rawreq->v.bat.umask & 0577);	/* Set partial umask */
		if ((rawreq->v.bat.stdout_acc & OMD_SPOOL) == 0) {
			/*
			 *  The standard-output file is to be accessed
			 *  directly.  We had to wait until after the chdir()
			 *  because the pathname could be relative.
			 */
			while ((standout = open (rawreq->v.bat.stdout_name,
				O_WRONLY | filemask | O_APPEND, 0777)) == -1 &&
				errno == EINTR)
				;
			if (standout == -1) {
				/*
				 *  Unable to create standard output file.
				 */
				if (errno == ENFILE) {
					/*
					 *  The system file table is full.
					 */
					serexit (RCM_ENFILERUN, (char *) 0);
				}
				if (errno == ENOSPC) {
					/*
					 *  Insufficient space.
					 */
					serexit (RCM_ENOSPCRUN, (char *) 0);
				}
				serexit (mergertcm (RCM_UNCRESTDOUT,
						    errnototcm()), (char *) 0);
			}
		}
		if ((rawreq->v.bat.stderr_acc & OMD_SPOOL) == 0 &&
		    (rawreq->v.bat.stderr_acc & OMD_EO) == 0) {
			/*
			 *  The standard-error file is to be accessed
			 *  directly.  We had to wait until after the
			 *  chdir() because the pathname may have been
			 *  specified as a relative pathname.
			 */
			while ((standerr = open (rawreq->v.bat.stderr_name,
				O_WRONLY | filemask | O_APPEND, 0777)) == -1 &&
				errno == EINTR)
				;
			if (standerr == -1) {
				/*
				 *  Unable to create standard error file.
				 */
				if (errno == ENFILE) {
					/*
					 *  The system file table is full.
					 */
					serexit (RCM_ENFILERUN, (char *) 0);
				}
				if (errno == ENOSPC) {
					/*
					 *  Insufficient space.
					 */
					serexit (RCM_ENOSPCRUN, (char *) 0);
				}
				serexit (mergertcm (RCM_UNCRESTDERR,
						    errnototcm()), (char *) 0);
			}
		}
		else if (rawreq->v.bat.stderr_acc & OMD_EO) {	/* -eo */
			/*
			 *  The stderr output of the request is to be directed
			 *  to the stdout file.
			 */
			standerr = fcntl (standout, F_DUPFD, 0);
			fcntl (standerr, F_SETFL, O_APPEND);
		}
		/*
		 *  Set the file creation mask to the value that it
		 *  had when the user first submitted the request.
		 */
		umask (rawreq->v.bat.umask);	/* Set precise umask */
		if (restartflag) {
			/*
			 *  This batch request is being restarted.
			 *  Write an appropriate message on the stdout
			 *  file indicating this fact.
			 */
			time (&timeval);	/* Get current time */
			sprintf (restart_msg,
			       "\n\n%%NQS(WARN): Request restarted at %s.\n\n",
				fmttime (&timeval));
			write (standout, restart_msg, strlen (restart_msg));
			if ((rawreq->v.bat.stderr_acc & OMD_EO) == 0) {
				/*
				 *  Write an appropriate message on the
				 *  stderr file as well.
				 */
				write (standerr, restart_msg,
				       strlen (restart_msg));
			}
		}
		/*
		 *  Mark the request as executing, if the request is
		 *  being executed for the first time.
		 *
		 *  WARNING:  We must do this now, BEFORE rearranging
		 *	      the file descriptor associations for the
		 *	      shell (see below).  Otherwise, file
		 *	      descriptor #3 will no longer be open
		 *	      as the request message/completion pipe
		 *	      back to the shepherd process.
		 */
		serexecute ();
		/*
		 *  Force the shell script file to be on file descriptor
		 *  #0.  Force the stdout and stderr files to be on file
	 	 *  descriptors #1 and #2 respectively.
		 */
		if (script != 0) {
			close (0);
			fcntl (script, F_DUPFD, 0);
			close (script);
		}
		if (standout != 1) {
			close (1);
			fcntl (standout, F_DUPFD, 1);
			close (standout);
		}
		if (standerr != 2) {
			close (2);
			fcntl (standerr, F_DUPFD, 2);
			close (standerr);
		}
		/*
		 *  Close all remaining file descriptors.
		 */
#if	BSD42 | BSD43 | ULTRIX
		i = getdtablesize();		/* #of file descrs per proc */
#else
#if	SGI | SYS52 | SYS53 | UNICOS | UTS
		i = _NFILE;			/* #of file descrs per proc */
#else
BAD SYSTEM TYPE
#endif
#endif
		while (--i >= 3) close (i);	/* Close all other descrs */
		/*
		 *  Exec the proper shell.
		 */
		server = defshell;	/* By default : Bourne shell */
		argv [0] = shellargs [0];
		argv [1] = shellargs [1];
		argv [2] = shellcmd;
#ifndef	NQS_LIBEXE
The symbol:  NQS_LIBEXE  MUST be defined from the Makefile.
#else
		sprintf (argv [2],	/* Shell command line */
#if	QSUB_SHELL
			"if (-r %s) source %s; %s/execsh; if (-r %s) source %s",
#else
			"if [ -r %s ]; then . %s; fi; %s/execsh; if [ -r %s ]; then . %s; fi",
#endif
			"/etc/prologue", "/etc/prologue", 
			NQS_LIBEXE,	/* Prologue and epilogue scripts */
			"/etc/epilogue", "/etc/epilogue"); 
#endif
		argv [3] = (char *)0;
		execve (server, argv, envp);
		/*
		 *  Fall through if execve() fails.
		 *
		 *  We have severed all connections to the shepherd
		 *  process that created us.  We must invoke a setuid
		 *  root program (shlexefai), to signal the shepherd
		 *  that the execve() of the shell, failed.
		 */
#ifndef	NQS_LIBEXE
The symbol:  NQS_LIBEXE  MUST be defined from the Makefile.
#else
		sprintf (buffer, "%s/shlexefai", NQS_LIBEXE);
#endif
		/*
		 *  Format errno as argv [1] for shlexefai.
		 */
		strcpy (argv0buffer, "shlexefai");
		sprintf (minusname, "%1d", errno);
		/*
		 *  Write shell execve() diagnostic on the stderr
		 *  file of the batch request.
		 */
		fprintf (stderr, "FATAL:  Execve() of shell: %s failed.\n",
			 server);
		fprintf (stderr, "INFO:   %s.\n", asciierrno());
		fflush (stdout);
		fflush (stderr);
		/*
		 *  Execve() shlexefai to inform the shepherd about
		 *  the shell execve() failure.
		 */
		argv [0] = argv0buffer;
		argv [1] = minusname;
		argv [2] = (char *) 0;
		envp [0] = (char *) 0;
		execve (buffer, argv, envp);
		/*
		 *  We are really having a bad day!  Nothing works!
		 *  Commit seppuku.  We cannot even display a diagnostic
		 *  to the NQS log file because our connection with the
		 *  NQS log process has been severed.
		 */
		kill (getpid(), SIGKILL);
	}
	/*
	 *  If the request is a device request, then the request must
	 *  be marked as executing.
	 */
	if (queue->q.type == QUE_DEVICE) serexecute ();
	/*
	 *  We are spawning a device, network, or pipe queue request.
	 *  Execve() the appropriate server.
	 */
	execve (server, argv, envp);
	/*
	 *  Execve() failed.
	 */
	serexit (RCM_SEREXEFAI, asciierrno());
}


/*** mkspool
 *
 *
 *	int mkspool():
 *
 *	Make a standard error or standard output file for a batch
 *	request in the NQS output spooling directory.
 *
 *	WARNING!!!!!!!!!!!!!!!!
 *		The EXTREMELY serious student will note that this
 *		function is invoked while the calling shell process
 *		is still running as root.
 *
 *		This is done because someday, NQS may be linked
 *		with the Newcastle distributed file access library.
 *		Since the Newcastle library converts ALL file paths
 *		to ABSOLUTE paths on the appropriate machine when
 *		invoking the requisite system call, NQS must still
 *		be running as root to safely create the spool output
 *		files in the spooling directory, since the spooling
 *		directory resides BELOW the NQS private directory,
 *		which only allows root access.
 *
 *	Returns:
 *	      >=0: if successful, as the opened file descriptor
 *		   for the stderr file;
 *	       -1: if an error occurs in which case errno is set.
 */
int mkspool (rawreq, namefn, passwd, filemask)
struct rawreq *rawreq;			/* Rawreq structure for request */
char *(*namefn)();			/* Ptr to fn returning ptr to char */
struct passwd *passwd;			/* Password file entry for owner */
int filemask;				/* File create/truncate flag */
{
	register int fd;		/* File descriptor */
	register char *path;		/* Ptr to temp file path */
	register int cmask;		/* umask */

	/*
	 *  Generate name of temporary spooled output file.
	 */
	path = (*namefn) (rawreq->orig_seqno, rawreq->orig_mid);
	/*
	 *  Set the file creation mask to the value that it
	 *  had when the user first submitted the request, with
	 *  the exception that the umask will NOT be allowed to
	 *  disable WRITE access for the request owner.
	 *
	 *  We do this so that the output return algorithm on
	 *  retry will not have to worry about being rebuffed
	 *  at the output file destination by a lack of write-
	 *  access.  (Note that the output return algorithm
	 *  replicates the permissions of the original file
	 *  for the destination file.)
	 */
	cmask = umask (rawreq->v.bat.umask & 0577);	/* Set partial umask */
	if ((fd = open (path, filemask | O_WRONLY | O_APPEND, 0777)) == -1) {
		return (-1);
	}
	umask (cmask);			/* Restore the file create mask */
	/*
	 *  Make sure that the user pays for the spooled disk space.
	 *  (The spool file was created while we were running as root).
	 */
	chown (path, passwd->pw_uid, passwd->pw_gid);
	return (fd);
}


/*** serexecute
 *
 *
 *	void serexecute():
 *
 *	Tell the shepherd process to set the request state to
 *	executing.
 */
void serexecute ()
{
	struct servermssg msg_packet;	/* Message packet */

	msg_packet.rcm = RCM_EXECUTING;
	msg_packet.id = jid;
	msg_packet.mssg [0] = '\0';
	/*
	 *  WARNING:	The file descriptor below (#3) must agree with
	 *		../src/nqs_spawn and ../lib/serexit.c.
	 */
	if (write (3, (char *) &msg_packet, sizeof (msg_packet)) !=
		sizeof (msg_packet)) {
		/*
		 * We cannot communicate with the shepherd through
		 * the pipe. Exit so the shepherd's read() will return.
		 */
		exit (0);
	}
}
