/*++ qlimit.c - Network Deviceing System
 *
 * $Source$
 *
 * DESCRIPTION:
 *
 *	Display  
 *	1) a list of the meaningful limits and
 *	2) the current shell strategy
 *	for the named machines.
 *	This program must run as a setuid root program.
 *
 *
 * RETURNS:
 *	
 * 	0	-  if output produced
 *	1	-  if an error occurred. A message is sent to the standard
 *			output file for every error.
 *
 *
 *	Author:
 *	-------
 *	Robert W. Sandstrom, Sterling Software Incorporated.
 *	January 13, 1986.
 *
 *
 * STANDARDS VIOLATIONS:
 *   None.
 *
 * REVISION HISTORY: ($Revision$ $Date$ $State$)
 * $Log$
 *
 */

#include <stdio.h>
#if	SGI | SYS52 | SYS53 | UNICOS | UTS
#include <fcntl.h>			/* File control */
#else
#if	BSD42 | BSD43 | ULTRIX
#include <sys/file.h>
#else
BAD SYSTEM TYPE
#endif
#endif
#include "nqs.h"
#include "nqsdirs.h"			/* NQS files and directories */


/*
 *	External functions:
 */
extern void bufstdout();		/* Block buffer stdout */
extern struct confd *opendb();		/* Open an NQS database file */
extern int shoalllim();			/* Show all limits on this machine */
extern int sholbymach();		/* Show limits given a machine name */

/*
 *	Global variables:
 */
char *Qlimit_prefix = "Qlimit";

/*** main
 *
 *
 *	qlimit [ <machine-name(s)> ]
 */
main (argc, argv)
int argc;
char *argv[];				/* Possible list of machines */
{
	struct confd *paramfile;
	int exitcode = 0;			/* Function exit code */

	if (chdir (Nqs_root) == -1) {
		fprintf (stderr, "%s(FATAL): Unable to chdir() to the NQS ",
			 Qlimit_prefix);
		fprintf (stderr, "root directory.\n");
		exit (1);
	}
	if ((paramfile = opendb (Nqs_params, O_RDONLY)) == NULL) {
		fprintf (stderr, "%s(FATAL): Unable to open the NQS ",
			 Qlimit_prefix);
		fprintf (stderr, "parameters database file.\n");
		exit (1);
	}
	/*
	 *  Block buffer stdout for efficiency.
	 */
	bufstdout();
	if (*++argv == NULL) exitcode |= shoalllim (paramfile, SHO_SHS);
	else {
		while (*argv != NULL) {		/* Show info on a machine */
			exitcode |= sholbymach (paramfile, *argv, SHO_SHS);
			argv++;
		}
	}
	/*
	 *  Flush output buffers and exit.
	 */
	fflush (stdout);
	fflush (stderr);
	exit (exitcode);
}
