/*++ qstat.c - Network Queueing System
 *
 * $Source$
 *
 * DESCRIPTION:
 *
 *	Display status information.
 *	This program must run as a setuid program.
 *
 *	qstat, qwatch, and qcmplx are very similiar commands
 *	the real difference between these commands are the 
 *	target item.  Qstat targets the queues.  Qwatch 
 *	targets the requests and qcmplx targets the complexes.
 *
 *	Since the method of arriving at the target displays are
 *	the same they share the same major sections of code.
 *	The arguments and target is passed onto main_dsp().
 *	This is ease in maintainability and readability.
 *
 *	The shared code may be found in lib/main_dsp.  Each
 *	set of code that is specific to queues, requests, and
 *	complexs may be found in queues.c, requests.c and complexes.c
 *	
 *	At one time a single binary with links for qstat, qwatch, and
 *	qcmplx was thought about.  We felt that this may constrain the
 *	code at some time in the future.  It was also felt that it may
 *	be more confusing to find the source code if it was the first
 *	time one had dove into the source.
 *
 *	Author:
 *	-------
 *	Terrie A. Carver, Computer Science Corperation.
 *	November 3, 1989
 *
 *
 * STANDARDS VIOLATIONS:
 *   None.
 *
 * REVISION HISTORY: ($Revision$ $Date$ $State$)
 * $Log$
 *
 */

#include <stdio.h>
#include "nqs.h"
#include "nqsdirs.h"			/* NQS files and directories */

/*** main
 *
 *
 *	qstat [-b] [-p] [-d] [-f] [-l] [-n] [-A] [-u] [-U] [-h] [-s]
 *		[ <queue-name(s)> ]
 */
main (argc, argv)
int argc;
char *argv[];
{
	long flags;
	char *selector = NULL;
	char *tname = NULL;
 	char *hname = NULL;

	/* Step 1. parse the command line */
	flags = parse_args (&argc, argv, "abdfh:lnps:u:A:U", &selector,
		&tname, &hname);

	/* Step 2. set SHO_QUE so we know the object to deal with  */

	/* If the options are -l, -b, -d, or -p, then we concerned */
	/* with the queues . . .                                   */

	if ((flags & SHO_BATCH) || (flags & SHO_DEVICE) || (flags & SHO_PIPE))
		flags |= SHO_QUE;

	/* If the options are -A acct, -U, -a */
	/* -s state, or -u user, . . . in other words, anything    */
	/* else (or nothing) . . . then we are concerned with      */
	/* requests.                                               */

	else
	  flags |= (SHO_BATCH | SHO_DEVICE | SHO_PIPE | SHO_REQ);

	
	/* Step 3. display the information */
	main_dsp (argc, argv, flags, selector, tname, hname);
	printf ("\n");
	exit (0);
}
